@@ -1,8 +1,8 @@
 r"""
 .. _broken-powerlaw-spectral-model:
 
-Broken Power Law Spectral Model
-======================================
+Broken power law spectral model
+===============================
 
 This model parametrises a broken power law spectrum.
 
@@ -24,11 +24,14 @@ from astropy import units as u
 import matplotlib.pyplot as plt
 from gammapy.modeling.models import BrokenPowerLawSpectralModel, Models, SkyModel
 
-energy_range = [0.1, 100] * u.TeV
+energy_bounds = [0.1, 100] * u.TeV
 model = BrokenPowerLawSpectralModel(
-    index1=1.5, index2=2.5, amplitude="1e-12 TeV-1 cm-2 s-1", ebreak="1 TeV",
+    index1=1.5,
+    index2=2.5,
+    amplitude="1e-12 TeV-1 cm-2 s-1",
+    ebreak="1 TeV",
 )
-model.plot(energy_range)
+model.plot(energy_bounds)
 plt.grid(which="both")
 
 # %%
