@@ -1,7 +1,7 @@
 r"""
 .. _super-exp-cutoff-powerlaw-4fgl-spectral-model:
 
-Super Exponential Cutoff Power Law Model used for 4FGL
+Super exponential cutoff power law model used for 4FGL
 ======================================================
 
 This model parametrises super exponential cutoff power-law model spectrum used for 4FGL.
@@ -30,7 +30,7 @@ from gammapy.modeling.models import (
     SuperExpCutoffPowerLaw4FGLSpectralModel,
 )
 
-energy_range = [0.1, 100] * u.TeV
+energy_bounds = [0.1, 100] * u.TeV
 model = SuperExpCutoffPowerLaw4FGLSpectralModel(
     index_1=1,
     index_2=2,
@@ -38,7 +38,7 @@ model = SuperExpCutoffPowerLaw4FGLSpectralModel(
     reference="1 TeV",
     expfactor=1e-2,
 )
-model.plot(energy_range)
+model.plot(energy_bounds)
 plt.grid(which="both")
 plt.ylim(1e-24, 1e-10)
 
