@@ -1,4 +1,4 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
-"""Gammapy high-level interface (analysis)."""
+"""Gammapy high level interface (analysis)."""
 from .config import *
 from .core import *
