@@ -8,7 +8,7 @@ from regions import CircleSkyRegion
 from pydantic.error_wrappers import ValidationError
 from gammapy.analysis import Analysis, AnalysisConfig
 from gammapy.datasets import MapDataset, SpectrumDatasetOnOff
-from gammapy.maps import Map, WcsNDMap
+from gammapy.maps import WcsGeom, WcsNDMap
 from gammapy.modeling.models import Models
 from gammapy.utils.testing import requires_data, requires_dependency
 
@@ -115,10 +115,21 @@ def test_get_observations_obs_time(tmp_path):
     analysis.get_observations()
     assert len(analysis.observations) == 40
     analysis.config.observations.obs_ids = [0]
-    with pytest.raises(ValueError):
+    with pytest.raises(KeyError):
         analysis.get_observations()
 
 
+@requires_data()
+def test_get_observations_missing_irf():
+    config = AnalysisConfig()
+    analysis = Analysis(config)
+    analysis.config.observations.datastore = "$GAMMAPY_DATA/joint-crab/dl3/magic/"
+    analysis.config.observations.obs_ids = ["05029748"]
+    analysis.config.observations.required_irf = ["aeff", "edisp"]
+    analysis.get_observations()
+    assert len(analysis.observations) == 1
+
+
 @requires_data()
 def test_set_models():
     config = get_example_config("1d")
@@ -127,7 +138,7 @@ def test_set_models():
     analysis.get_datasets()
     models_str = Path(MODEL_FILE).read_text()
     analysis.set_models(models=models_str)
-    assert isinstance(analysis.models, Models) is True
+    assert isinstance(analysis.models, Models)
     with pytest.raises(TypeError):
         analysis.set_models(0)
 
@@ -139,6 +150,10 @@ def test_analysis_1d():
     observations:
         datastore: $GAMMAPY_DATA/hess-dl3-dr1
         obs_ids: [23523, 23526]
+        obs_time: {
+            start: [J2004.92654346, J2004.92658453, J2004.92663655],
+            stop: [J2004.92658453, J2004.92663655, J2004.92670773]
+        }
     datasets:
         type: 1d
         background:
@@ -153,6 +168,12 @@ def test_analysis_1d():
         containment_correction: false
     flux_points:
         energy: {min: 1 TeV, max: 50 TeV, nbins: 4}
+    light_curve:
+        energy_edges: {min: 1 TeV, max: 50 TeV, nbins: 1}
+        time_intervals: {
+            start: [J2004.92654346, J2004.92658453, J2004.92663655],
+            stop: [J2004.92658453, J2004.92663655, J2004.92670773]
+        }
     """
     config = get_example_config("1d")
     analysis = Analysis(config)
@@ -162,14 +183,24 @@ def test_analysis_1d():
     analysis.read_models(MODEL_FILE_1D)
     analysis.run_fit()
     analysis.get_flux_points()
+    analysis.get_light_curve()
 
-    assert len(analysis.datasets) == 2
-    assert len(analysis.flux_points.data.table) == 4
-    dnde = analysis.flux_points.data.table["dnde"].quantity
+    assert len(analysis.datasets) == 3
+    table = analysis.flux_points.data.to_table(sed_type="dnde")
+
+    assert len(table) == 4
+    dnde = table["dnde"].quantity
     assert dnde.unit == "cm-2 s-1 TeV-1"
 
     assert_allclose(dnde[0].value, 8.116854e-12, rtol=1e-2)
-    assert_allclose(dnde[2].value, 3.547128e-14, rtol=1e-2)
+    assert_allclose(dnde[2].value, 3.444475e-14, rtol=1e-2)
+
+    axis = analysis.light_curve.geom.axes["time"]
+    assert axis.nbin == 3
+    assert_allclose(axis.time_min.mjd, [53343.92, 53343.935, 53343.954])
+
+    flux = analysis.light_curve.flux.data[:, :, 0, 0]
+    assert_allclose(flux, [[1.688954e-11], [2.347870e-11], [1.604152e-11]], rtol=1e-4)
 
 
 @requires_data()
@@ -209,10 +240,10 @@ def test_geom_analysis_1d():
 def test_exclusion_region(tmp_path):
     config = get_example_config("1d")
     analysis = Analysis(config)
-    exclusion_region = CircleSkyRegion(center=SkyCoord("85d 23d"), radius=1 * u.deg)
-    exclusion_mask = Map.create(npix=(150, 150), binsz=0.05, skydir=SkyCoord("83d 22d"))
-    mask = exclusion_mask.geom.region_mask([exclusion_region], inside=False)
-    exclusion_mask.data = mask.astype(int)
+    region = CircleSkyRegion(center=SkyCoord("85d 23d"), radius=1 * u.deg)
+    geom = WcsGeom.create(npix=(150, 150), binsz=0.05, skydir=SkyCoord("83d 22d"))
+    exclusion_mask = ~geom.region_mask([region])
+
     filename = tmp_path / "exclusion.fits"
     exclusion_mask.write(filename)
     config.datasets.background.method = "reflected"
@@ -226,10 +257,9 @@ def test_exclusion_region(tmp_path):
     analysis.get_observations()
     analysis.get_datasets()
     geom = analysis.datasets[0]._geom
-    exclusion = WcsNDMap.from_geom(geom)
-    exclusion.data = geom.region_mask([exclusion_region], inside=False).astype(int)
+    exclusion_mask = ~geom.region_mask([region])
     filename = tmp_path / "exclusion3d.fits"
-    exclusion.write(filename)
+    exclusion_mask.write(filename)
     config.datasets.background.exclusion = filename
     analysis.get_datasets()
     assert len(analysis.datasets) == 1
@@ -237,17 +267,26 @@ def test_exclusion_region(tmp_path):
 
 @requires_dependency("iminuit")
 @requires_data()
-def test_analysis_1d_stacked():
+def test_analysis_1d_stacked_no_fit_range():
     cfg = """
+    observations:
+        datastore: $GAMMAPY_DATA/hess-dl3-dr1
+        obs_cone: {frame: icrs, lon: 83.633 deg, lat: 22.014 deg, radius: 5 deg}
+        obs_ids: [23592, 23559]
+
     datasets:
+        type: 1d
+        stack: false
         geom:
             axes:
+                energy: {min: 0.01 TeV, max: 100 TeV, nbins: 73}
                 energy_true: {min: 0.03 TeV, max: 100 TeV, nbins: 50}
+        on_region: {frame: icrs, lon: 83.633 deg, lat: 22.014 deg, radius: 0.1 deg}
+        containment_correction: true
         background:
             method: reflected
     """
-
-    config = get_example_config("1d")
+    config = AnalysisConfig.from_yaml(cfg)
     analysis = Analysis(config)
     analysis.update_config(cfg)
     analysis.config.datasets.stack = True
@@ -255,13 +294,16 @@ def test_analysis_1d_stacked():
     analysis.get_datasets()
     analysis.read_models(MODEL_FILE_1D)
     analysis.run_fit()
+    with pytest.raises(ValueError):
+        analysis.get_excess_map()
 
     assert len(analysis.datasets) == 1
     assert_allclose(analysis.datasets["stacked"].counts.data.sum(), 184)
-    pars = analysis.fit_result.parameters
+    pars = analysis.models.parameters
+    assert_allclose(analysis.datasets[0].mask_fit.data, True)
 
     assert_allclose(pars["index"].value, 2.76913, rtol=1e-2)
-    assert_allclose(pars["amplitude"].value, 5.496388e-11, rtol=1e-2)
+    assert_allclose(pars["amplitude"].value, 5.479729e-11, rtol=1e-2)
 
 
 @requires_data()
@@ -273,10 +315,13 @@ def test_analysis_ring_background():
     analysis = Analysis(config)
     analysis.get_observations()
     analysis.get_datasets()
+    analysis.get_excess_map()
     assert isinstance(analysis.datasets[0], MapDataset)
     assert_allclose(
         analysis.datasets[0].npred_background().data[0, 10, 10], 0.091799, rtol=1e-2
     )
+    assert isinstance(analysis.excess_map["sqrt_ts"], WcsNDMap)
+    assert_allclose(analysis.excess_map.npred_excess.data[0, 62, 62], 134.12389)
 
 
 @requires_data()
@@ -291,19 +336,30 @@ def test_analysis_ring_3d():
 
 
 @requires_data()
-def test_analysis_no_bkg():
+def test_analysis_no_bkg_1d(caplog):
     config = get_example_config("1d")
     analysis = Analysis(config)
     analysis.get_observations()
     analysis.get_datasets()
-    assert isinstance(analysis.datasets[0], SpectrumDatasetOnOff) is False
+    assert not isinstance(analysis.datasets[0], SpectrumDatasetOnOff)
+    assert "WARNING" in [_.levelname for _ in caplog.records]
+    assert "No background maker set. Check configuration." in [
+        _.message for _ in caplog.records
+    ]
 
+
+@requires_data()
+def test_analysis_no_bkg_3d(caplog):
     config = get_example_config("3d")
     config.datasets.background.method = None
     analysis = Analysis(config)
     analysis.get_observations()
     analysis.get_datasets()
-    assert isinstance(analysis.datasets[0], MapDataset) is True
+    assert isinstance(analysis.datasets[0], MapDataset)
+    assert "WARNING" in [_.levelname for _ in caplog.records]
+    assert "No background maker set. Check configuration." in [
+        _.message for _ in caplog.records
+    ]
 
 
 @requires_dependency("iminuit")
@@ -319,14 +375,16 @@ def test_analysis_3d():
     analysis.get_flux_points()
 
     assert len(analysis.datasets) == 1
-    assert len(analysis.fit_result.parameters) == 8
-    res = analysis.fit_result.parameters
+    assert len(analysis.models.parameters) == 8
+    res = analysis.models.parameters
     assert res["amplitude"].unit == "cm-2 s-1 TeV-1"
-    assert len(analysis.flux_points.data.table) == 2
-    dnde = analysis.flux_points.data.table["dnde"].quantity
 
-    assert_allclose(dnde[0].value, 1.376879e-11, rtol=1e-2)
-    assert_allclose(dnde[-1].value, 2.691466e-13, rtol=1e-2)
+    table = analysis.flux_points.data.to_table(sed_type="dnde")
+    assert len(table) == 2
+    dnde = table["dnde"].quantity
+
+    assert_allclose(dnde[0].value, 1.339052e-11, rtol=1e-2)
+    assert_allclose(dnde[-1].value, 2.772374e-13, rtol=1e-2)
     assert_allclose(res["index"].value, 3.097613, rtol=1e-2)
     assert_allclose(res["tilt"].value, -0.207792, rtol=1e-2)
 
