@@ -7,7 +7,7 @@ from .gammacat import *
 from .hawc import *
 from .hess import *
 
-SOURCE_CATALOGS = Registry(
+CATALOG_REGISTRY = Registry(
     [
         SourceCatalogGammaCat,
         SourceCatalogHGPS,
@@ -16,23 +16,26 @@ SOURCE_CATALOGS = Registry(
         SourceCatalog4FGL,
         SourceCatalog2FHL,
         SourceCatalog3FHL,
+        SourceCatalog3HWC,
     ]
 )
 """Registry of source catalogs in Gammapy."""
 
+
 __all__ = [
-    "SOURCE_CATALOGS",
+    "CATALOG_REGISTRY",
     "SourceCatalog",
-    "SourceCatalogObject",
     "SourceCatalogObjectHGPS",
     "SourceCatalogObject2FHL",
     "SourceCatalogObject3FHL",
     "SourceCatalogObject3FGL",
     "SourceCatalogObject4FGL",
     "SourceCatalogObject2HWC",
+    "SourceCatalogObject3HWC",
     "SourceCatalogObjectGammaCat",
     "SourceCatalogObjectHGPSComponent",
     "SourceCatalogLargeScaleHGPS",
+    "SourceCatalogObject",
 ]
 
-__all__.extend(cls.__name__ for cls in SOURCE_CATALOGS)
+__all__.extend(cls.__name__ for cls in CATALOG_REGISTRY)
