@@ -4,10 +4,10 @@
 # no matter how it is invoked within the source tree.
 import os
 from astropy.tests.helper import enable_deprecations_as_exceptions
-from astropy.tests.plugins.display import PYTEST_HEADER_MODULES
+from pytest_astropy_header.display import PYTEST_HEADER_MODULES
 
-# TODO: add numpy again once https://github.com/astropy/regions/pull/252 is addressed
-enable_deprecations_as_exceptions(warnings_to_ignore_entire_module=["numpy", "astropy"])
+# TODO: activate this again and handle deprecations in the code
+# enable_deprecations_as_exceptions(warnings_to_ignore_entire_module=["iminuit", "naima"])
 
 # Declare for which packages version numbers should be displayed
 # when running the tests
@@ -19,13 +19,14 @@ PYTEST_HEADER_MODULES["healpy"] = "healpy"
 PYTEST_HEADER_MODULES["sherpa"] = "sherpa"
 PYTEST_HEADER_MODULES["gammapy"] = "gammapy"
 PYTEST_HEADER_MODULES["naima"] = "naima"
-PYTEST_HEADER_MODULES["reproject"] = "reproject"
 
 
 def pytest_configure(config):
     """Print some info ..."""
     from gammapy.utils.testing import has_data
 
+    config.option.astropy_header = True
+
     print("")
     print("Gammapy test data availability:")
 
