@@ -2,6 +2,7 @@
 import logging
 import subprocess
 from pathlib import Path
+from astropy import units as u
 from astropy.coordinates import SkyCoord
 from astropy.io import fits
 from gammapy.utils.scripts import make_path
@@ -38,6 +39,19 @@ class DataStore:
     >>> from gammapy.data import DataStore
     >>> data_store = DataStore.from_dir('$GAMMAPY_DATA/hess-dl3-dr1')
     >>> data_store.info()
+    Data store:
+    HDU index table:
+    BASE_DIR: /home/runner/work/gammapy/gammapy/gammapy-datasets/hess-dl3-dr1
+    Rows: 630
+    OBS_ID: 20136 -- 47829
+    HDU_TYPE: ['aeff', 'bkg', 'edisp', 'events', 'gti', 'psf']
+    HDU_CLASS: ['aeff_2d', 'bkg_3d', 'edisp_2d', 'events', 'gti', 'psf_table']
+    <BLANKLINE>
+    <BLANKLINE>
+    Observation table:
+    Observatory name: 'N/A'
+    Number of observations: 105
+    <BLANKLINE>
     """
 
     DEFAULT_HDU_TABLE = "hdu-index.fits.gz"
@@ -67,6 +81,11 @@ class DataStore:
             FITS HDU name or number for the HDU index table
         hdu_obs : str or int
             FITS HDU name or number for the observation index table
+
+        Returns
+        -------
+        data_store : `DataStore`
+            Data store
         """
         filename = make_path(filename)
 
@@ -204,27 +223,17 @@ class DataStore:
             raise ValueError(f"OBS_ID = {obs_id} not in HDU index table.")
 
         row = self.obs_table.select_obs_id(obs_id=obs_id)[0]
-        obs_info = table_row_to_dict(row)
+        kwargs = {"obs_id": int(obs_id)}
+        kwargs["obs_info"] = table_row_to_dict(row)
 
-        aeff_hdu = self.hdu_table.hdu_location(obs_id=obs_id, hdu_type="aeff")
-        edisp_hdu = self.hdu_table.hdu_location(obs_id=obs_id, hdu_type="edisp")
-        bkg_hdu = self.hdu_table.hdu_location(obs_id=obs_id, hdu_type="bkg")
-        psf_hdu = self.hdu_table.hdu_location(obs_id=obs_id, hdu_type="psf")
-        events_hdu = self.hdu_table.hdu_location(obs_id=obs_id, hdu_type="events")
-        gti_hdu = self.hdu_table.hdu_location(obs_id=obs_id, hdu_type="gti")
+        hdu_list = ["events", "gti", "aeff", "edisp", "psf", "bkg", "rad_max"]
 
-        return Observation(
-            obs_id=int(obs_id),
-            obs_info=obs_info,
-            bkg=bkg_hdu,
-            aeff=aeff_hdu,
-            edisp=edisp_hdu,
-            events=events_hdu,
-            gti=gti_hdu,
-            psf=psf_hdu,
-        )
+        for hdu in hdu_list:
+            kwargs[hdu] = self.hdu_table.hdu_location(obs_id=obs_id, hdu_type=hdu)
 
-    def get_observations(self, obs_id=None, skip_missing=False):
+        return Observation(**kwargs)
+
+    def get_observations(self, obs_id=None, skip_missing=False, required_irf="all"):
         """Generate a `~gammapy.data.Observations`.
 
         Parameters
@@ -233,16 +242,39 @@ class DataStore:
             Observation IDs (default of ``None`` means "all")
         skip_missing : bool, optional
             Skip missing observations, default: False
+        required_irf : list of str
+            Runs will be added to the list of observations only if the
+            required IRFs are present. Otherwise, the given run will be skipped
+            Available options are:
+            * `aeff` : Effective area
+            * `bkg` : Background
+            * `edisp`: Energy dispersion
+            * `psf` : Point Spread Function
+            By default, all the IRFs are required.
 
         Returns
         -------
         observations : `~gammapy.data.Observations`
             Container holding a list of `~gammapy.data.Observation`
         """
+        available_irf = ["aeff", "edisp", "psf", "bkg"]
+
+        if required_irf == "all":
+            required_irf = available_irf
+        elif required_irf is None:
+            required_irf = []
+
+        if not set(required_irf).issubset(available_irf):
+            difference = set(required_irf).difference(available_irf)
+            raise ValueError(
+                f"{difference} is not a valid irf key. Choose from: {available_irf}"
+            )
+
         if obs_id is None:
             obs_id = self.obs_table["OBS_ID"].data
 
         obs_list = []
+
         for _ in obs_id:
             try:
                 obs = self.obs(_)
@@ -252,8 +284,12 @@ class DataStore:
                     continue
                 else:
                     raise err
-            else:
+
+            if set(required_irf).issubset(obs.available_irfs):
                 obs_list.append(obs)
+            else:
+                log.warning(f"Skipping run with missing IRFs; obs_id: {_!r}")
+
         return Observations(obs_list)
 
     def copy_obs(self, obs_id, outdir, hdu_class=None, verbose=False, overwrite=False):
@@ -419,7 +455,9 @@ class DataStoreMaker:
 
     @staticmethod
     def read_events_info(path):
+        """Read mandatory events header info"""
         log.debug(f"Reading {path}")
+
         with fits.open(path, memmap=False) as hdu_list:
             header = hdu_list["EVENTS"].header
 
@@ -427,20 +465,28 @@ class DataStoreMaker:
 
         info = {}
         # Note: for some reason `header["OBS_ID"]` is sometimes `str`, maybe trailing whitespace
+        # mandatory header info:
         info["OBS_ID"] = int(header["OBS_ID"])
-        info["RA_PNT"] = header["RA_PNT"]
-        info["DEC_PNT"] = header["DEC_PNT"]
-        pos = SkyCoord(info["RA_PNT"], info["DEC_PNT"], unit="deg").galactic
-        info["GLON_PNT"] = pos.l.deg
-        info["GLAT_PNT"] = pos.b.deg
-        info["ZEN_PNT"] = 90 - float(header["ALT_PNT"])
-        info["ALT_PNT"] = header["ALT_PNT"]
-        info["AZ_PNT"] = header["AZ_PNT"]
-        info["ONTIME"] = header["ONTIME"]
-        info["LIVETIME"] = header["LIVETIME"]
+        info["TSTART"] = header["TSTART"] * u.s
+        info["TSTOP"] = header["TSTOP"] * u.s
+        info["ONTIME"] = header["ONTIME"] * u.s
+        info["LIVETIME"] = header["LIVETIME"] * u.s
         info["DEADC"] = header["DEADC"]
-        info["TSTART"] = header["TSTART"]
-        info["TSTOP"] = header["TSTOP"]
+        info["TELESCOP"] = header.get("TELESCOP", na_str)
+
+        obs_mode = header.get("OBS_MODE", "POINTING")
+        if obs_mode == "DRIFT":
+            info["ALT_PNT"] = header["ALT_PNT"] * u.deg
+            info["AZ_PNT"] = header["AZ_PNT"] * u.deg
+            info["ZEN_PNT"] = 90 * u.deg - info["ALT_PNT"]
+        else:
+            info["RA_PNT"] = header["RA_PNT"] * u.deg
+            info["DEC_PNT"] = header["DEC_PNT"] * u.deg
+
+        # optional header info
+        pos = SkyCoord(info["RA_PNT"], info["DEC_PNT"], unit="deg").galactic
+        info["GLON_PNT"] = pos.l
+        info["GLAT_PNT"] = pos.b
         info["DATE-OBS"] = header.get("DATE_OBS", na_str)
         info["TIME-OBS"] = header.get("TIME_OBS", na_str)
         info["DATE-END"] = header.get("DATE_END", na_str)
@@ -449,18 +495,12 @@ class DataStoreMaker:
         info["OBJECT"] = header.get("OBJECT", na_str)
 
         # This is the info needed to link from EVENTS to IRFs
-        info["TELESCOP"] = header.get("TELESCOP", na_str)
         info["CALDB"] = header.get("CALDB", na_str)
         info["IRF"] = header.get("IRF", na_str)
 
         # Not part of the spec, but good to know from which file the info comes
         info["EVENTS_FILENAME"] = str(path)
         info["EVENT_COUNT"] = header["NAXIS2"]
-
-        # gti = Table.read(filename, hdu='GTI')
-        # info['GTI_START'] = gti['START'][0]
-        # info['GTI_STOP'] = gti['STOP'][0]
-
         return info
 
     def make_obs_table(self):
@@ -472,18 +512,6 @@ class DataStoreMaker:
         names = list(rows[0].keys())
         table = ObservationTable(rows=rows, names=names)
 
-        table["RA_PNT"].unit = "deg"
-        table["DEC_PNT"].unit = "deg"
-        table["GLON_PNT"].unit = "deg"
-        table["GLAT_PNT"].unit = "deg"
-        table["ZEN_PNT"].unit = "deg"
-        table["ALT_PNT"].unit = "deg"
-        table["AZ_PNT"].unit = "deg"
-        table["ONTIME"].unit = "s"
-        table["LIVETIME"].unit = "s"
-        table["TSTART"].unit = "s"
-        table["TSTOP"].unit = "s"
-
         # TODO: Values copied from one of the EVENTS headers
         # TODO: check consistency for all EVENTS files and handle inconsistent case
         # Transform times to first ref time? Or raise error for now?
