@@ -5,7 +5,7 @@ from astropy.coordinates import SkyCoord
 from astropy.table import Table
 from regions import CircleSkyRegion, RectangleSkyRegion
 from gammapy.data import EventList
-from gammapy.maps import Map, MapAxis, WcsGeom
+from gammapy.maps import MapAxis, WcsGeom
 from gammapy.utils.testing import mpl_plot_check, requires_data, requires_dependency
 
 
@@ -192,9 +192,8 @@ class TestEventSelection:
             skydir=(0, 0), binsz=0.2, width=4.0 * u.deg, proj="TAN", axes=[axis]
         )
 
-        mask_data = geom.region_mask(regions=[self.on_regions[0]], inside=True)
-        mask = Map.from_geom(geom, data=mask_data)
-        new_list = self.events.select_map_mask(mask)
+        mask = geom.region_mask(regions=[self.on_regions[0]])
+        new_list = self.events.select_mask(mask)
         assert len(new_list.table) == 2
 
     def test_select_energy(self):
