@@ -40,7 +40,7 @@ def test_hdu_index_table(hdu_index_table):
     See https://github.com/open-gamma-ray-astro/gamma-astro-data-formats/issues/118
     """
     location = hdu_index_table.hdu_location(obs_id=42, hdu_type="events")
-    assert location.path().as_posix() == "spam/a/b"
+    assert location.path().as_posix() == "a/b"
 
     location = hdu_index_table.hdu_location(obs_id=42, hdu_type="bkg")
     assert location is None
@@ -49,7 +49,7 @@ def test_hdu_index_table(hdu_index_table):
 
 
 @requires_data()
-def test_hdu_index_table_hd_hap():
+def test_hdu_index_table_hd_hap(capfd):
     """Test HESS HAP-HD data access."""
     hdu_index = HDUIndexTable.read("$GAMMAPY_DATA/hess-dl3-dr1/hdu-index.fits.gz")
 
@@ -59,6 +59,13 @@ def test_hdu_index_table_hd_hap():
     # A few valid queries
 
     location = hdu_index.hdu_location(obs_id=23523, hdu_type="events")
+
+    # We test here the std out
+    location.info()
+    out, err = capfd.readouterr()
+    lines = out.split("\n")
+    assert len(lines) == 6
+
     hdu = location.get_hdu()
     assert hdu.name == "EVENTS"
 
@@ -83,6 +90,11 @@ def test_hdu_index_table_hd_hap():
         "data/hess_dl3_dr1_obs_id_023523.fits.gz"
     )
 
+    location = hdu_index.hdu_location(obs_id=23523, hdu_type="bkg")
+    assert location.path(abs_path=False) == Path(
+        "data/hess_dl3_dr1_obs_id_023523.fits.gz"
+    )
+
     # A few invalid queries
 
     with pytest.raises(IndexError) as excinfo:
@@ -97,10 +109,16 @@ def test_hdu_index_table_hd_hap():
 
     with pytest.raises(ValueError) as excinfo:
         hdu_index.hdu_location(obs_id=23523, hdu_type="invalid")
-    msg = "Invalid hdu_type: invalid. Valid values are: ['events', 'gti', 'aeff', 'edisp', 'psf', 'bkg']"
+    msg = "Invalid hdu_type: invalid. Valid values are: ['events', 'gti', 'aeff', 'edisp', 'psf', 'bkg', 'rad_max']"
     assert str(excinfo.value) == msg
 
     with pytest.raises(ValueError) as excinfo:
         hdu_index.hdu_location(obs_id=23523, hdu_class="invalid")
-    msg = "Invalid hdu_class: invalid. Valid values are: ['events', 'gti', 'aeff_2d', 'edisp_2d', 'psf_table', 'psf_3gauss', 'psf_king', 'bkg_2d', 'bkg_3d']"
+    msg = "Invalid hdu_class: invalid. Valid values are: ['events', 'gti', 'aeff_2d', 'edisp_2d', 'psf_table', 'psf_3gauss', 'psf_king', 'bkg_2d', 'bkg_3d', 'rad_max_2d']"
     assert str(excinfo.value) == msg
+
+    location = hdu_index.hdu_location(obs_id=23523, hdu_class="psf_king")
+    assert location is None
+
+    location = hdu_index.hdu_location(obs_id=23523, hdu_class="bkg_2d")
+    assert location is None
