@@ -40,6 +40,16 @@ class Dataset(abc.ABC):
     def tag(self):
         pass
 
+    @property
+    def name(self):
+        return self._name
+
+    def to_dict(self):
+        """Convert to dict for YAML serialization."""
+        name = self.name.replace(" ", "_")
+        filename = f"{name}.fits"
+        return {"name": self.name, "type": self.tag, "filename": filename}
+
     @property
     def mask(self):
         """Combined fit and safe mask"""
@@ -64,18 +74,23 @@ class Dataset(abc.ABC):
         """Statistic array, one value per data point."""
 
     def copy(self, name=None):
-        """A deep copy."""
+        """A deep copy.
+
+        Parameters
+        ----------
+        name : str
+            Name of the copied dataset
+
+        Returns
+        -------
+        dataset : `Dataset`
+            Copied datasets.
+        """
         new = copy.deepcopy(self)
         name = make_name(name)
         new._name = name
-
-        # propagate new dataset name
-        if new._models is not None:
-            for m in new._models:
-                if m.datasets_names is not None:
-                    for k, d in enumerate(m.datasets_names):
-                        if d == self.name:
-                            m.datasets_names[k] = name
+        # TODO: check the model behaviour?
+        new.models = None
         return new
 
     @staticmethod
@@ -89,8 +104,7 @@ class Dataset(abc.ABC):
                 residuals = (data - model) / np.sqrt(model)
             else:
                 raise AttributeError(
-                    f"Invalid method: {method!r}. Choose between 'diff',"
-                    " 'diff/model' and 'diff/sqrt(model)'"
+                    f"Invalid method: {method!r} for computing residuals"
                 )
         return residuals
 
@@ -183,6 +197,25 @@ class Datasets(collections.abc.MutableSequence):
         axes = [d.counts.geom.axes["energy"] for d in self]
         return np.all([axes[0].is_aligned(ax) for ax in axes])
 
+    @property
+    def contributes_to_stat(self):
+        """Stat contributions
+
+        Returns
+        -------
+        contributions : `~numpy.array`
+            Array indicating which dataset contributes to the likelihood.
+        """
+        contributions = []
+
+        for dataset in self:
+            if dataset.mask is not None:
+                value = dataset.mask.data.any()
+            else:
+                value = True
+            contributions.append(value)
+        return np.array(contributions)
+
     def stat_sum(self):
         """Compute joint likelihood"""
         stat_sum = 0
@@ -191,12 +224,12 @@ class Datasets(collections.abc.MutableSequence):
             stat_sum += dataset.stat_sum()
         return stat_sum
 
-    def select_time(self, t_min, t_max, atol="1e-6 s"):
+    def select_time(self, time_min, time_max, atol="1e-6 s"):
         """Select datasets in a given time interval.
 
         Parameters
         ----------
-        t_min, t_max : `~astropy.time.Time`
+        time_min, time_max : `~astropy.time.Time`
             Time interval
         atol : `~astropy.units.Quantity`
             Tolerance value for time comparison with different scale. Default 1e-6 sec.
@@ -215,7 +248,7 @@ class Datasets(collections.abc.MutableSequence):
             t_start = dataset.gti.time_start[0]
             t_stop = dataset.gti.time_stop[-1]
 
-            if t_start >= (t_min - atol) and t_stop <= (t_max + atol):
+            if t_start >= (time_min - atol) and t_stop <= (time_max + atol):
                 datasets.append(dataset)
 
         return self.__class__(datasets)
@@ -223,6 +256,9 @@ class Datasets(collections.abc.MutableSequence):
     def slice_by_energy(self, energy_min, energy_max):
         """Select and slice datasets in energy range
 
+        The method keeps the current dataset names. Datasets, that do not
+        contribute to the selected energy range are dismissed.
+
         Parameters
         ----------
         energy_min, energy_max : `~astropy.units.Quantity`
@@ -241,7 +277,7 @@ class Datasets(collections.abc.MutableSequence):
                 dataset_sliced = dataset.slice_by_energy(
                     energy_min=energy_min,
                     energy_max=energy_max,
-                    name=dataset.name + "-slice",
+                    name=dataset.name,
                 )
             except ValueError:
                 log.info(
@@ -253,6 +289,29 @@ class Datasets(collections.abc.MutableSequence):
 
         return self.__class__(datasets=datasets)
 
+    def to_spectrum_datasets(self, region):
+        """Extract spectrum datasets for the given region.
+
+        Parameters
+        ----------
+        region : `~regions.SkyRegion`
+            Region definition.
+
+        Returns
+        --------
+        datasets : `Datasets`
+            List of `~gammapy.datasets.SpectrumDataset`
+        """
+        datasets = Datasets()
+
+        for dataset in self:
+            spectrum_dataset = dataset.to_spectrum_dataset(
+                on_region=region, name=dataset.name
+            )
+            datasets.append(spectrum_dataset)
+
+        return datasets
+
     @property
     # TODO: make this a method to support different methods?
     def energy_ranges(self):
@@ -354,38 +413,47 @@ class Datasets(collections.abc.MutableSequence):
         filename : str or `Path`
             File path or name of datasets yaml file
         filename_models : str or `Path`
-            File path or name of models fyaml ile
+            File path or name of models yaml file
         overwrite : bool
             overwrite datasets FITS files
         write_covariance : bool
             save covariance or not
         """
-        path = make_path(filename).resolve()
+        path = make_path(filename)
 
         data = {"datasets": []}
 
         for dataset in self._datasets:
-            name = dataset.name.replace(" ", "_")
-            filename = f"{name}.fits"
+            d = dataset.to_dict()
+            filename = d["filename"]
             dataset.write(path.parent / filename, overwrite=overwrite)
-            data["datasets"].append(dataset.to_dict(filename=filename))
+            data["datasets"].append(d)
 
         write_yaml(data, path, sort_keys=False)
 
         if filename_models:
             self.models.write(
-                filename_models, overwrite=overwrite, write_covariance=write_covariance,
+                filename_models,
+                overwrite=overwrite,
+                write_covariance=write_covariance,
             )
 
-    def stack_reduce(self, name=None):
+    def stack_reduce(self, name=None, nan_to_num=True):
         """Reduce the Datasets to a unique Dataset by stacking them together.
 
         This works only if all Dataset are of the same type and if a proper
         in-place stack method exists for the Dataset type.
 
+        Parameters
+        ----------
+        name : str
+            Name of the stacked dataset.
+        nan_to_num: bool
+            Non-finite values are replaced by zero if True (default).
+
         Returns
         -------
-        dataset : ~gammapy.utils.Dataset
+        dataset : `~gammapy.datasets.Dataset`
             the stacked dataset
         """
         if not self.is_all_same_type:
@@ -393,12 +461,14 @@ class Datasets(collections.abc.MutableSequence):
                 "Stacking impossible: all Datasets contained are not of a unique type."
             )
 
-        dataset = self[0].copy(name=name)
-        for ds in self[1:]:
-            dataset.stack(ds)
-        return dataset
+        stacked = self[0].to_masked(name=name, nan_to_num=nan_to_num)
 
-    def info_table(self, cumulative=False, region=None):
+        for dataset in self[1:]:
+            stacked.stack(dataset, nan_to_num=nan_to_num)
+
+        return stacked
+
+    def info_table(self, cumulative=False):
         """Get info table for datasets.
 
         Parameters
@@ -414,7 +484,7 @@ class Datasets(collections.abc.MutableSequence):
         if not self.is_all_same_type:
             raise ValueError("Info table not supported for mixed dataset type.")
 
-        stacked = self[0].copy(name=self[0].name)
+        stacked = self[0].to_masked(name="stacked")
 
         rows = [stacked.info_dict()]
 
@@ -436,10 +506,13 @@ class Datasets(collections.abc.MutableSequence):
         time_intervals = []
 
         for dataset in self:
-            if dataset.gti is not None:
+            if dataset.gti is not None and len(dataset.gti.table) > 0:
                 interval = (dataset.gti.time_start[0], dataset.gti.time_stop[-1])
                 time_intervals.append(interval)
 
+        if len(time_intervals) == 0:
+            return None
+
         return GTI.from_time_intervals(time_intervals)
 
     @property
