@@ -7,14 +7,10 @@ from gammapy.datasets import Datasets, FluxPointsDataset
 from gammapy.estimators import FluxPoints
 from gammapy.modeling import Fit
 from gammapy.modeling.models import PowerLawSpectralModel, SkyModel
+from gammapy.utils.scripts import make_path
 from gammapy.utils.testing import mpl_plot_check, requires_data, requires_dependency
 
 
-@pytest.fixture()
-def fit(dataset):
-    return Fit([dataset])
-
-
 @pytest.fixture()
 def test_meta_table(dataset):
     meta_table = dataset.meta_table
@@ -26,8 +22,9 @@ def test_meta_table(dataset):
 @pytest.fixture()
 def dataset():
     path = "$GAMMAPY_DATA/tests/spectrum/flux_points/diff_flux_points.fits"
-    data = FluxPoints.read(path)
-    data.table["e_ref"] = data.energy_ref.to("TeV")
+    table = Table.read(make_path(path))
+    table["e_ref"] = table["e_ref"].quantity.to("TeV")
+    data = FluxPoints.from_table(table, format="gadf-sed")
     model = SkyModel(
         spectral_model=PowerLawSpectralModel(
             index=2.3, amplitude="2e-13 cm-2 s-1 TeV-1", reference="1 TeV"
@@ -46,8 +43,10 @@ def dataset():
 @requires_data()
 def test_flux_point_dataset_serialization(tmp_path):
     path = "$GAMMAPY_DATA/tests/spectrum/flux_points/diff_flux_points.fits"
-    data = FluxPoints.read(path)
-    data.table["e_ref"] = data.energy_ref.to("TeV")
+    table = Table.read(make_path(path))
+    table["e_ref"] = table["e_ref"].quantity.to("TeV")
+    data = FluxPoints.from_table(table, format="gadf-sed")
+
     spectral_model = PowerLawSpectralModel(
         index=2.3, amplitude="2e-13 cm-2 s-1 TeV-1", reference="1 TeV"
     )
@@ -65,7 +64,7 @@ def test_flux_point_dataset_serialization(tmp_path):
     )
 
     new_dataset = datasets[0]
-    assert_allclose(new_dataset.data.table["dnde"], dataset.data.table["dnde"], 1e-4)
+    assert_allclose(new_dataset.data.dnde, dataset.data.dnde, 1e-4)
     if dataset.mask_fit is None:
         assert np.all(new_dataset.mask_fit == dataset.mask_safe)
     assert np.all(new_dataset.mask_safe == dataset.mask_safe)
@@ -80,53 +79,74 @@ def test_flux_point_dataset_str(dataset):
 @requires_data()
 class TestFluxPointFit:
     @requires_dependency("iminuit")
-    def test_fit_pwl_minuit(self, fit):
-        result = fit.run(backend="minuit")
-        self.assert_result(result)
+    def test_fit_pwl_minuit(self, dataset):
+        fit = Fit()
+        result = fit.run(dataset)
+        self.assert_result(result, dataset.models)
 
     @requires_dependency("sherpa")
-    def test_fit_pwl_sherpa(self, fit):
-        result = fit.optimize(backend="sherpa", method="simplex")
-        self.assert_result(result)
+    def test_fit_pwl_sherpa(self, dataset):
+        fit = Fit(backend="sherpa", optimize_opts={"method": "simplex"})
+        result = fit.optimize(datasets=[dataset])
+        self.assert_result(result, dataset.models)
 
     @staticmethod
-    def assert_result(result):
+    def assert_result(result, models):
         assert result.success
         assert_allclose(result.total_stat, 25.2059, rtol=1e-3)
 
-        index = result.parameters["index"]
+        index = models.parameters["index"]
         assert_allclose(index.value, 2.216, rtol=1e-3)
 
-        amplitude = result.parameters["amplitude"]
+        amplitude = models.parameters["amplitude"]
         assert_allclose(amplitude.value, 2.1616e-13, rtol=1e-3)
 
-        reference = result.parameters["reference"]
+        reference = models.parameters["reference"]
         assert_allclose(reference.value, 1, rtol=1e-8)
 
     @staticmethod
     @requires_dependency("iminuit")
-    def test_stat_profile(fit):
+    def test_stat_profile(dataset):
+        fit = Fit()
+        result = fit.run(datasets=dataset)
 
-        result = fit.run(backend="minuit")
+        model = dataset.models[0].spectral_model
 
-        profile = fit.stat_profile("amplitude", nvalues=3, bounds=1)
+        assert_allclose(model.amplitude.error, 1.9e-14, rtol=1e-2)
+
+        model.amplitude.scan_n_values = 3
+        model.amplitude.scan_n_sigma = 1
+        model.amplitude.interp = "lin"
+
+        profile = fit.stat_profile(
+            datasets=dataset,
+            parameter="amplitude",
+        )
 
         ts_diff = profile["stat_scan"] - result.total_stat
-        assert_allclose(ts_diff, [110.1, 0, 110.1], rtol=1e-2, atol=1e-7)
+        assert_allclose(
+            model.amplitude.scan_values, [1.97e-13, 2.16e-13, 2.35e-13], rtol=1e-2
+        )
+        assert_allclose(ts_diff, [110.244116, 0.0, 110.292074], rtol=1e-2, atol=1e-7)
 
-        value = result.parameters["amplitude"].value
-        err = result.parameters["amplitude"].error
-        values = np.array([value - err, value, value + err])
+        value = model.parameters["amplitude"].value
+        err = model.parameters["amplitude"].error
 
-        profile = fit.stat_profile("amplitude", values=values)
+        model.amplitude.scan_values = np.array([value - err, value, value + err])
+        profile = fit.stat_profile(
+            datasets=dataset,
+            parameter="amplitude",
+        )
 
         ts_diff = profile["stat_scan"] - result.total_stat
-        assert_allclose(ts_diff, [110.1, 0, 110.1], rtol=1e-2, atol=1e-7)
+        assert_allclose(
+            model.amplitude.scan_values, [1.97e-13, 2.16e-13, 2.35e-13], rtol=1e-2
+        )
+        assert_allclose(ts_diff, [110.244116, 0.0, 110.292074], rtol=1e-2, atol=1e-7)
 
     @staticmethod
     @requires_dependency("matplotlib")
-    def test_fp_dataset_plot_fit(fit):
-        fp_dataset = fit.datasets[0]
+    def test_fp_dataset_plot_fit(dataset):
 
         with mpl_plot_check():
-            fp_dataset.plot_fit(kwargs_residuals=dict(method="diff/model"))
+            dataset.plot_fit(kwargs_residuals=dict(method="diff/model"))
