@@ -11,7 +11,10 @@ def test_datasets_to_io(tmp_path):
     filedata = "$GAMMAPY_DATA/tests/models/gc_example_datasets.yaml"
     filemodel = "$GAMMAPY_DATA/tests/models/gc_example_models.yaml"
 
-    datasets = Datasets.read(filename=filedata, filename_models=filemodel,)
+    datasets = Datasets.read(
+        filename=filedata,
+        filename_models=filemodel,
+    )
 
     assert len(datasets) == 2
     assert len(datasets.models) == 5
@@ -67,4 +70,4 @@ def test_datasets_to_io(tmp_path):
     assert_allclose(dataset0.npred_background().data.sum(), 15726.8, atol=0.1)
 
     dataset_copy = dataset0.copy(name="dataset0-copy")
-    assert dataset_copy.background_model.datasets_names == ["dataset0-copy"]
+    assert dataset_copy.models is None
