@@ -7,7 +7,8 @@ from astropy.table import Table
 from astropy.time import Time
 from gammapy.data import GTI
 from gammapy.datasets import Datasets, SpectrumDataset, SpectrumDatasetOnOff
-from gammapy.irf import EDispKernelMap, EffectiveAreaTable
+from gammapy.irf import EDispKernelMap, EffectiveAreaTable2D
+from gammapy.makers.utils import make_map_exposure_true_energy
 from gammapy.maps import MapAxis, RegionGeom, RegionNDMap, WcsGeom
 from gammapy.modeling import Fit
 from gammapy.modeling.models import (
@@ -19,7 +20,7 @@ from gammapy.modeling.models import (
     SkyModel,
 )
 from gammapy.utils.random import get_random_state
-from gammapy.utils.regions import compound_region_to_list, make_region
+from gammapy.utils.regions import compound_region_to_regions
 from gammapy.utils.testing import (
     assert_time_allclose,
     mpl_plot_check,
@@ -36,7 +37,9 @@ def spectrum_dataset():
     livetime = 100 * u.s
 
     pwl = PowerLawSpectralModel(
-        index=2.1, amplitude="1e5 cm-2 s-1 TeV-1", reference="0.1 TeV",
+        index=2.1,
+        amplitude="1e5 cm-2 s-1 TeV-1",
+        reference="0.1 TeV",
     )
 
     temp_mod = ConstantTemporalModel()
@@ -59,7 +62,11 @@ def spectrum_dataset():
     gti = GTI.create(start, stop, reference_time=t_ref)
 
     dataset = SpectrumDataset(
-        models=models, exposure=exposure, background=background, name=name, gti=gti,
+        models=models,
+        exposure=exposure,
+        background=background,
+        name=name,
+        gti=gti,
     )
     dataset.fake(random_state=23)
     return dataset
@@ -74,9 +81,11 @@ def test_str(spectrum_dataset):
 
 
 def test_energy_range(spectrum_dataset):
-    energy_range = spectrum_dataset.energy_range
-    assert energy_range.unit == u.TeV
-    assert_allclose(energy_range.to_value("TeV"), [0.1, 10.0])
+    e_min, e_max = spectrum_dataset.energy_range
+    assert e_min.unit == u.TeV
+    assert e_max.unit == u.TeV
+    assert_allclose(e_min, 0.1)
+    assert_allclose(e_max, 10.0)
 
 
 def test_info_dict(spectrum_dataset):
@@ -92,14 +101,6 @@ def test_info_dict(spectrum_dataset):
     assert info_dict["name"] == "test"
 
 
-def test_incorrect_mask(spectrum_dataset):
-    mask_fit = np.ones(30, dtype=np.dtype("float"))
-    with pytest.raises(ValueError):
-        SpectrumDataset(
-            counts=spectrum_dataset.counts.copy(), mask_fit=mask_fit,
-        )
-
-
 def test_set_model(spectrum_dataset):
     spectrum_dataset = spectrum_dataset.copy()
     spectral_model = PowerLawSpectralModel()
@@ -114,7 +115,10 @@ def test_set_model(spectrum_dataset):
 
 def test_npred_models():
     e_reco = MapAxis.from_energy_bounds("1 TeV", "10 TeV", nbin=3)
-    spectrum_dataset = SpectrumDataset.create(e_reco=e_reco)
+
+    geom = RegionGeom(region=None, axes=[e_reco])
+
+    spectrum_dataset = SpectrumDataset.create(geom=geom)
     spectrum_dataset.exposure.quantity = 1e10 * u.Unit("cm2 h")
 
     pwl_1 = PowerLawSpectralModel(index=2)
@@ -130,24 +134,37 @@ def test_npred_models():
     npred_sig = spectrum_dataset.npred_signal()
     assert_allclose(npred_sig.data.sum(), 64.8)
 
-    npred_sig_model1 = spectrum_dataset.npred_signal(model=model_1)
+    npred_sig_model1 = spectrum_dataset.npred_signal(model_name=model_1.name)
     assert_allclose(npred_sig_model1.data.sum(), 32.4)
 
 
+def test_npred_spatial_model(spectrum_dataset):
+    model = SkyModel.create("pl", "gauss", name="test")
+
+    spectrum_dataset.models = [model]
+
+    npred = spectrum_dataset.npred()
+    model.spatial_model.sigma.value = 1.0
+    npred_large_sigma = spectrum_dataset.npred()
+
+    assert_allclose(npred.data.sum(), 3000)
+    assert_allclose(npred_large_sigma.data.sum(), 3000)
+    assert spectrum_dataset.evaluators["test"].psf is None
+
+
 @requires_dependency("iminuit")
 def test_fit(spectrum_dataset):
     """Simple CASH fit to the on vector"""
-    fit = Fit([spectrum_dataset])
-    result = fit.run()
-
-    # assert result.success
+    fit = Fit()
+    result = fit.run(datasets=[spectrum_dataset])
+    assert result.success
     assert "minuit" in repr(result)
 
     npred = spectrum_dataset.npred().data.sum()
     assert_allclose(npred, 907012.186399, rtol=1e-3)
     assert_allclose(result.total_stat, -18087404.624, rtol=1e-3)
 
-    pars = result.parameters
+    pars = spectrum_dataset.models.parameters
     assert_allclose(pars["index"].value, 2.1, rtol=1e-2)
     assert_allclose(pars["index"].error, 0.001276, rtol=1e-2)
 
@@ -160,18 +177,21 @@ def test_spectrum_dataset_create():
     e_true = MapAxis.from_edges(
         u.Quantity([0.05, 0.5, 5, 20.0], "TeV"), name="energy_true"
     )
-    empty_spectrum_dataset = SpectrumDataset.create(e_reco, e_true, name="test")
+    geom = RegionGeom(region=None, axes=[e_reco])
+    empty_spectrum_dataset = SpectrumDataset.create(
+        geom, energy_axis_true=e_true, name="test"
+    )
 
     assert empty_spectrum_dataset.name == "test"
     assert empty_spectrum_dataset.counts.data.sum() == 0
     assert empty_spectrum_dataset.data_shape[0] == 2
-    assert empty_spectrum_dataset.npred_background().data.sum() == 0
-    assert empty_spectrum_dataset.npred_background().geom.axes[0].nbin == 2
+    assert empty_spectrum_dataset.background.data.sum() == 0
+    assert empty_spectrum_dataset.background.geom.axes[0].nbin == 2
     assert empty_spectrum_dataset.exposure.geom.axes[0].nbin == 3
     assert empty_spectrum_dataset.edisp.edisp_map.geom.axes["energy"].nbin == 2
     assert empty_spectrum_dataset.gti.time_sum.value == 0
     assert len(empty_spectrum_dataset.gti.table) == 0
-    assert empty_spectrum_dataset.energy_range[0] is None
+    assert np.isnan(empty_spectrum_dataset.energy_range[0])
     assert_allclose(empty_spectrum_dataset.mask_safe, 0)
 
 
@@ -183,41 +203,47 @@ def test_spectrum_dataset_stack_diagonal_safe_mask(spectrum_dataset):
         "0.1 TeV", "10 TeV", nbin=30, name="energy_true"
     )
 
-    aeff = EffectiveAreaTable.from_parametrization(energy.edges, "HESS").to_region_map(
-        geom.region
+    aeff = EffectiveAreaTable2D.from_parametrization(
+        energy_axis_true=energy_true, instrument="HESS"
     )
 
     livetime = 100 * u.s
     gti = GTI.create(start=0 * u.s, stop=livetime)
 
-    exposure = aeff * livetime
+    geom_true = geom.as_energy_true
+    exposure = make_map_exposure_true_energy(
+        geom=geom_true, livetime=livetime, pointing=geom_true.center_skydir, aeff=aeff
+    )
 
     edisp = EDispKernelMap.from_diagonal_response(
         energy, energy_true, geom=geom.to_image()
     )
     edisp.exposure_map.data = exposure.data[:, :, np.newaxis, :]
 
-    background = spectrum_dataset.npred_background().copy()
+    background = spectrum_dataset.background
+
+    mask_safe = RegionNDMap.from_geom(geom=geom, dtype=bool)
+    mask_safe.data += True
 
     spectrum_dataset1 = SpectrumDataset(
         name="ds1",
         counts=spectrum_dataset.counts.copy(),
         exposure=exposure.copy(),
         edisp=edisp.copy(),
-        background=background,
+        background=background.copy(),
         gti=gti.copy(),
+        mask_safe=mask_safe,
     )
 
     livetime2 = 0.5 * livetime
     gti2 = GTI.create(start=200 * u.s, stop=200 * u.s + livetime2)
-    aeff2 = aeff * 2
     bkg2 = RegionNDMap.from_geom(geom=geom, data=2 * background.data)
 
     geom = spectrum_dataset.counts.geom
     data = np.ones(spectrum_dataset.data_shape, dtype="bool")
     data[0] = False
     safe_mask2 = RegionNDMap.from_geom(geom=geom, data=data)
-    exposure2 = aeff2 * livetime2
+    exposure2 = exposure.copy()
 
     edisp = edisp.copy()
     edisp.exposure_map.data = exposure2.data[:, :, np.newaxis, :]
@@ -236,16 +262,12 @@ def test_spectrum_dataset_stack_diagonal_safe_mask(spectrum_dataset):
     reference = spectrum_dataset.counts.data
     assert_allclose(spectrum_dataset1.counts.data[1:], reference[1:] * 2)
     assert_allclose(spectrum_dataset1.counts.data[0], 141363)
-    assert_allclose(spectrum_dataset1.exposure.data[0], 4.755644e09)
     assert_allclose(
-        spectrum_dataset1.npred_background().data[1:], 3 * background.data[1:]
+        spectrum_dataset1.exposure.quantity[0], 4.755644e09 * u.Unit("cm2 s")
     )
-    assert_allclose(spectrum_dataset1.npred_background().data[0], background.data[0])
+    assert_allclose(spectrum_dataset1.background.data[1:], 3 * background.data[1:])
+    assert_allclose(spectrum_dataset1.background.data[0], background.data[0])
 
-    assert_allclose(
-        spectrum_dataset1.exposure.quantity.to_value("m2s"),
-        2 * (aeff * livetime).quantity.to_value("m2s"),
-    )
     kernel = edisp.get_edisp_kernel()
     kernel_stacked = spectrum_dataset1.edisp.get_edisp_kernel()
 
@@ -254,36 +276,55 @@ def test_spectrum_dataset_stack_diagonal_safe_mask(spectrum_dataset):
 
 
 def test_spectrum_dataset_stack_nondiagonal_no_bkg(spectrum_dataset):
-    energy = spectrum_dataset.counts.geom.axes[0]
-    geom = spectrum_dataset.counts.geom.to_image()
+    energy = spectrum_dataset.counts.geom.axes["energy"]
+    geom = spectrum_dataset.counts.geom
 
-    edisp1 = EDispKernelMap.from_gauss(energy, energy, 0.1, 0, geom=geom)
+    edisp1 = EDispKernelMap.from_gauss(
+        energy_axis=energy,
+        energy_axis_true=energy.copy(name="energy_true"),
+        sigma=0.1,
+        bias=0,
+        geom=geom.to_image(),
+    )
     edisp1.exposure_map.data += 1
 
-    aeff = EffectiveAreaTable.from_parametrization(energy.edges, "HESS").to_region_map(
-        geom.region
+    aeff = EffectiveAreaTable2D.from_parametrization(
+        energy_axis_true=energy.copy(name="energy_true"), instrument="HESS"
+    )
+
+    livetime = 100 * u.s
+
+    geom_true = geom.as_energy_true
+    exposure = make_map_exposure_true_energy(
+        geom=geom_true, livetime=livetime, pointing=geom_true.center_skydir, aeff=aeff
     )
 
     geom = spectrum_dataset.counts.geom
     counts = RegionNDMap.from_geom(geom=geom)
 
-    gti = GTI.create(start=0 * u.s, stop=100 * u.s)
+    gti = GTI.create(start=0 * u.s, stop=livetime)
     spectrum_dataset1 = SpectrumDataset(
         counts=counts,
-        exposure=aeff * gti.time_sum,
+        exposure=exposure,
         edisp=edisp1,
         meta_table=Table({"OBS_ID": [0]}),
         gti=gti.copy(),
     )
 
-    edisp2 = EDispKernelMap.from_gauss(energy, energy, 0.2, 0.0, geom=geom)
+    edisp2 = EDispKernelMap.from_gauss(
+        energy_axis=energy,
+        energy_axis_true=energy.copy(name="energy_true"),
+        sigma=0.2,
+        bias=0.0,
+        geom=geom,
+    )
     edisp2.exposure_map.data += 1
 
     gti2 = GTI.create(start=100 * u.s, stop=200 * u.s)
 
     spectrum_dataset2 = SpectrumDataset(
         counts=counts,
-        exposure=aeff * gti2.time_sum,
+        exposure=exposure.copy(),
         edisp=edisp2,
         meta_table=Table({"OBS_ID": [1]}),
         gti=gti2,
@@ -316,14 +357,13 @@ def test_peek(spectrum_dataset):
 
 
 class TestSpectrumOnOff:
-    """ Test ON OFF SpectrumDataset"""
+    """Test ON OFF SpectrumDataset"""
 
     def setup(self):
         etrue = np.logspace(-1, 1, 10) * u.TeV
         self.e_true = MapAxis.from_energy_edges(etrue, name="energy_true")
         ereco = np.logspace(-1, 1, 5) * u.TeV
         elo = ereco[:-1]
-        ehi = ereco[1:]
         self.e_reco = MapAxis.from_energy_edges(ereco, name="energy")
 
         start = u.Quantity([0], "s")
@@ -332,15 +372,13 @@ class TestSpectrumOnOff:
         self.gti = GTI.create(start, stop, time_ref)
         self.livetime = self.gti.time_sum
 
-        self.on_region = make_region("icrs;circle(0.,1.,0.1)")
-        off_region = make_region("icrs;box(0.,1.,0.1, 0.2,30)")
-        self.off_region = off_region.union(
-            make_region("icrs;box(-1.,-1.,0.1, 0.2,150)")
-        )
         self.wcs = WcsGeom.create(npix=300, binsz=0.01, frame="icrs").wcs
 
         self.aeff = RegionNDMap.create(
-            region=self.on_region, wcs=self.wcs, axes=[self.e_true], unit="cm2"
+            region="icrs;circle(0.,1.,0.1)",
+            wcs=self.wcs,
+            axes=[self.e_true],
+            unit="cm2",
         )
         self.aeff.data += 1
 
@@ -349,7 +387,7 @@ class TestSpectrumOnOff:
 
         axis = MapAxis.from_edges(ereco, name="energy", interp="log")
         self.on_counts = RegionNDMap.create(
-            region=self.on_region,
+            region="icrs;circle(0.,1.,0.1)",
             wcs=self.wcs,
             axes=[axis],
             meta={"EXPOSURE": self.livetime.to_value("s")},
@@ -358,7 +396,9 @@ class TestSpectrumOnOff:
         self.on_counts.data[-1] = 0
 
         self.off_counts = RegionNDMap.create(
-            region=self.off_region, wcs=self.wcs, axes=[axis]
+            region="icrs;box(0.,1.,0.1, 0.2,30);box(-1.,-1.,0.1, 0.2,150)",
+            wcs=self.wcs,
+            axes=[axis],
         )
         self.off_counts.data += 10
 
@@ -378,6 +418,9 @@ class TestSpectrumOnOff:
         exposure = self.aeff * self.livetime
         exposure.meta["livetime"] = self.livetime
 
+        mask_safe = RegionNDMap.from_geom(self.on_counts.geom, dtype=bool)
+        mask_safe.data += True
+
         self.dataset = SpectrumDatasetOnOff(
             counts=self.on_counts,
             counts_off=self.off_counts,
@@ -387,6 +430,7 @@ class TestSpectrumOnOff:
             acceptance_off=acceptance_off,
             name="test",
             gti=self.gti,
+            mask_safe=mask_safe,
         )
 
     def test_spectrum_dataset_on_off_create(self):
@@ -394,25 +438,32 @@ class TestSpectrumOnOff:
         e_true = MapAxis.from_edges(
             u.Quantity([0.05, 0.5, 5, 20.0], "TeV"), name="energy_true"
         )
-        empty_dataset = SpectrumDatasetOnOff.create(e_reco, e_true)
+        geom = RegionGeom(region=None, axes=[e_reco])
+        empty_dataset = SpectrumDatasetOnOff.create(geom=geom, energy_axis_true=e_true)
 
         assert empty_dataset.counts.data.sum() == 0
         assert empty_dataset.data_shape[0] == 2
         assert empty_dataset.counts_off.data.sum() == 0
         assert empty_dataset.counts_off.geom.axes[0].nbin == 2
-        assert_allclose(empty_dataset.acceptance_off, 1)
-        assert_allclose(empty_dataset.acceptance, 1)
+        assert_allclose(empty_dataset.acceptance_off, 0)
+        assert_allclose(empty_dataset.acceptance, 0)
         assert empty_dataset.acceptance.data.shape[0] == 2
         assert empty_dataset.acceptance_off.data.shape[0] == 2
         assert empty_dataset.gti.time_sum.value == 0
         assert len(empty_dataset.gti.table) == 0
-        assert empty_dataset.energy_range[0] is None
+        assert np.isnan(empty_dataset.energy_range[0])
 
     def test_create_stack(self):
-        stacked = SpectrumDatasetOnOff.create(self.e_reco, self.e_true)
+        geom = RegionGeom(region=None, axes=[self.e_reco])
+
+        stacked = SpectrumDatasetOnOff.create(geom=geom, energy_axis_true=self.e_true)
+        stacked.mask_safe.data += True
 
         stacked.stack(self.dataset)
-        assert_allclose(stacked.energy_range.value, self.dataset.energy_range.value)
+        e_min_stacked, e_max_stacked = stacked.energy_range
+        e_min_dataset, e_max_dataset = self.dataset.energy_range
+        assert_allclose(e_min_stacked, e_min_dataset)
+        assert_allclose(e_max_stacked, e_max_dataset)
 
     def test_alpha(self):
         assert self.dataset.alpha.data.shape == (4, 1, 1)
@@ -424,7 +475,7 @@ class TestSpectrumOnOff:
         livetime = 1 * u.s
 
         aeff = RegionNDMap.create(
-            region=self.on_region,
+            region=self.aeff.geom.region,
             unit="cm2",
             axes=[self.e_reco.copy(name="energy_true")],
         )
@@ -445,7 +496,7 @@ class TestSpectrumOnOff:
         ds = self.dataset.to_spectrum_dataset()
 
         assert isinstance(ds, SpectrumDataset)
-        assert_allclose(ds.npred_background().data.sum(), 4)
+        assert_allclose(ds.background.data.sum(), 4)
 
     @requires_dependency("matplotlib")
     def test_peek(self):
@@ -472,11 +523,15 @@ class TestSpectrumOnOff:
 
     def test_to_from_ogip_files(self, tmp_path):
         dataset = self.dataset.copy(name="test")
-        dataset.to_ogip_files(outdir=tmp_path)
-        newdataset = SpectrumDatasetOnOff.from_ogip_files(tmp_path / "pha_obstest.fits")
+        dataset.write(tmp_path / "test.fits")
+        newdataset = SpectrumDatasetOnOff.read(tmp_path / "test.fits")
 
-        expected_regions = compound_region_to_list(self.off_counts.geom.region)
-        regions = compound_region_to_list(newdataset.counts_off.geom.region)
+        expected_regions = compound_region_to_regions(self.off_counts.geom.region)
+        regions = compound_region_to_regions(newdataset.counts_off.geom.region)
+
+        assert newdataset.counts.meta["RESPFILE"] == "test_rmf.fits"
+        assert newdataset.counts.meta["BACKFILE"] == "test_bkg.fits"
+        assert newdataset.counts.meta["ANCRFILE"] == "test_arf.fits"
 
         assert_allclose(self.on_counts.data, newdataset.counts.data)
         assert_allclose(self.off_counts.data, newdataset.counts_off.data)
@@ -487,11 +542,30 @@ class TestSpectrumOnOff:
         assert regions[0].center.is_equivalent_frame(expected_regions[0].center)
         assert_allclose(regions[1].angle, expected_regions[1].angle)
 
+    def test_to_from_ogip_files_no_mask(self, tmp_path):
+        dataset = self.dataset.copy(name="test")
+        dataset.mask_safe = None
+        dataset.write(tmp_path / "test.fits")
+        newdataset = SpectrumDatasetOnOff.read(tmp_path / "test.fits")
+
+        assert_allclose(newdataset.mask_safe.data, True)
+
+    def test_to_from_ogip_files_zip(self, tmp_path):
+        dataset = self.dataset.copy(name="test")
+        dataset.write(tmp_path / "test.fits.gz")
+        newdataset = SpectrumDatasetOnOff.read(tmp_path / "test.fits.gz")
+
+        assert newdataset.counts.meta["RESPFILE"] == "test_rmf.fits.gz"
+        assert newdataset.counts.meta["BACKFILE"] == "test_bkg.fits.gz"
+        assert newdataset.counts.meta["ANCRFILE"] == "test_arf.fits.gz"
+
     def test_to_from_ogip_files_no_edisp(self, tmp_path):
 
         mask_safe = RegionNDMap.from_geom(self.on_counts.geom, dtype=bool)
         mask_safe.data += True
 
+        acceptance = RegionNDMap.from_geom(self.on_counts.geom, data=1.0)
+
         exposure = self.aeff * self.livetime
         exposure.meta["livetime"] = self.livetime
 
@@ -499,11 +573,11 @@ class TestSpectrumOnOff:
             counts=self.on_counts,
             exposure=exposure,
             mask_safe=mask_safe,
-            acceptance=1,
+            acceptance=acceptance,
             name="test",
         )
-        dataset.to_ogip_files(outdir=tmp_path)
-        newdataset = SpectrumDatasetOnOff.from_ogip_files(tmp_path / "pha_obstest.fits")
+        dataset.write(tmp_path / "pha_obstest.fits")
+        newdataset = SpectrumDatasetOnOff.read(tmp_path / "pha_obstest.fits")
 
         assert_allclose(self.on_counts.data, newdataset.counts.data)
         assert newdataset.counts_off is None
@@ -515,15 +589,15 @@ class TestSpectrumOnOff:
             energy_min=0.3 * u.TeV, energy_max=6 * u.TeV
         )
         desired = [False, True, True, False]
-        assert_allclose(mask[:, 0, 0], desired)
+        assert_allclose(mask.data[:, 0, 0], desired)
 
         mask = self.dataset.counts.geom.energy_mask(energy_max=6 * u.TeV)
         desired = [True, True, True, False]
-        assert_allclose(mask[:, 0, 0], desired)
+        assert_allclose(mask.data[:, 0, 0], desired)
 
         mask = self.dataset.counts.geom.energy_mask(energy_min=1 * u.TeV)
         desired = [False, False, True, True]
-        assert_allclose(mask[:, 0, 0], desired)
+        assert_allclose(mask.data[:, 0, 0], desired)
 
     def test_str(self):
         model = SkyModel(spectral_model=PowerLawSpectralModel())
@@ -533,8 +607,8 @@ class TestSpectrumOnOff:
             models=model,
             exposure=self.aeff * self.livetime,
             edisp=self.edisp,
-            acceptance=1,
-            acceptance_off=10,
+            acceptance=RegionNDMap.from_geom(geom=self.on_counts.geom, data=1),
+            acceptance_off=RegionNDMap.from_geom(geom=self.off_counts.geom, data=10),
         )
         assert "SpectrumDatasetOnOff" in str(dataset)
         assert "wstat" in str(dataset)
@@ -549,8 +623,8 @@ class TestSpectrumOnOff:
             models=source_model,
             exposure=self.aeff * self.livetime,
             edisp=self.edisp,
-            acceptance=1,
-            acceptance_off=10,
+            acceptance=RegionNDMap.from_geom(geom=self.on_counts.geom, data=1),
+            acceptance_off=RegionNDMap.from_geom(geom=self.off_counts.geom, data=10),
         )
         real_dataset = dataset.copy()
 
@@ -613,11 +687,10 @@ class TestSpectralFit:
         path = "$GAMMAPY_DATA/joint-crab/spectra/hess/"
         self.datasets = Datasets(
             [
-                SpectrumDatasetOnOff.from_ogip_files(path + "pha_obs23523.fits"),
-                SpectrumDatasetOnOff.from_ogip_files(path + "pha_obs23592.fits"),
+                SpectrumDatasetOnOff.read(path + "pha_obs23523.fits"),
+                SpectrumDatasetOnOff.read(path + "pha_obs23592.fits"),
             ]
         )
-
         self.pwl = SkyModel(
             spectral_model=PowerLawSpectralModel(
                 index=2, amplitude=1e-12 * u.Unit("cm-2 s-1 TeV-1"), reference=1 * u.TeV
@@ -635,7 +708,7 @@ class TestSpectralFit:
 
         # Example fit for one observation
         self.datasets[0].models = self.pwl
-        self.fit = Fit([self.datasets[0]])
+        self.fit = Fit()
 
     def set_model(self, model):
         for obs in self.datasets:
@@ -644,8 +717,8 @@ class TestSpectralFit:
     @requires_dependency("iminuit")
     def test_basic_results(self):
         self.set_model(self.pwl)
-        result = self.fit.run()
-        pars = self.fit.datasets.parameters
+        result = self.fit.run([self.datasets[0]])
+        pars = self.datasets.parameters
 
         assert self.pwl is self.datasets[0].models[0]
 
@@ -658,8 +731,8 @@ class TestSpectralFit:
 
     def test_basic_errors(self):
         self.set_model(self.pwl)
-        result = self.fit.run()
-        pars = result.parameters
+        result = self.fit.run([self.datasets[0]])
+        pars = self.pwl.parameters
 
         assert_allclose(pars["index"].error, 0.149633, rtol=1e-3)
         assert_allclose(pars["amplitude"].error, 6.423139e-12, rtol=1e-3)
@@ -667,21 +740,21 @@ class TestSpectralFit:
 
     def test_ecpl_fit(self):
         self.set_model(self.ecpl)
-        fit = Fit([self.datasets[0]])
-        fit.run()
+        fit = Fit()
+        fit.run([self.datasets[0]])
 
-        actual = fit.datasets.parameters["lambda_"].quantity
+        actual = self.datasets.parameters["lambda_"].quantity
         assert actual.unit == "TeV-1"
         assert_allclose(actual.value, 0.145215, rtol=1e-2)
 
     def test_joint_fit(self):
         self.set_model(self.pwl)
-        fit = Fit(self.datasets)
-        fit.run()
-        actual = fit.datasets.parameters["index"].value
+        fit = Fit()
+        fit.run(self.datasets)
+        actual = self.datasets.parameters["index"].value
         assert_allclose(actual, 2.7806, rtol=1e-3)
 
-        actual = fit.datasets.parameters["amplitude"].quantity
+        actual = self.datasets.parameters["amplitude"].quantity
         assert actual.unit == "cm-2 s-1 TeV-1"
         assert_allclose(actual.value, 5.200e-11, rtol=1e-3)
 
@@ -689,9 +762,8 @@ class TestSpectralFit:
         dataset = self.datasets[0].copy()
         dataset.models = self.pwl
 
-        fit = Fit([dataset])
-        result = fit.run()
-
+        fit = Fit()
+        result = fit.run(datasets=[dataset])
         stats = dataset.stat_array()
         actual = np.sum(stats[dataset.mask_safe])
 
@@ -699,12 +771,12 @@ class TestSpectralFit:
         assert_allclose(actual, desired)
 
     def test_fit_range(self):
-        # Fit range not restriced fit range should be the thresholds
+        # Fit range not restricted fit range should be the thresholds
         obs = self.datasets[0]
         actual = obs.energy_range[0]
 
         assert actual.unit == "keV"
-        assert_allclose(actual.value, 8.912509e08)
+        assert_allclose(actual, 8.912509e08)
 
     def test_no_edisp(self):
         dataset = self.datasets[0].copy()
@@ -712,18 +784,18 @@ class TestSpectralFit:
         dataset.edisp = None
         dataset.models = self.pwl
 
-        fit = Fit([dataset])
-        result = fit.run()
-        assert_allclose(result.parameters["index"].value, 2.7961, atol=0.02)
+        fit = Fit()
+        result = fit.run(datasets=[dataset])
+        assert_allclose(self.pwl.spectral_model.index.value, 2.7961, atol=0.02)
 
     def test_stacked_fit(self):
         dataset = self.datasets[0].copy()
         dataset.stack(self.datasets[1])
         dataset.models = SkyModel(PowerLawSpectralModel())
 
-        fit = Fit([dataset])
-        result = fit.run()
-        pars = result.parameters
+        fit = Fit()
+        result = fit.run(datasets=[dataset])
+        pars = dataset.models.parameters
 
         assert_allclose(pars["index"].value, 2.7767, rtol=1e-3)
         assert u.Unit(pars["amplitude"].unit) == "cm-2 s-1 TeV-1"
@@ -732,8 +804,8 @@ class TestSpectralFit:
 
 def _read_hess_obs():
     path = "$GAMMAPY_DATA/joint-crab/spectra/hess/"
-    obs1 = SpectrumDatasetOnOff.from_ogip_files(path + "pha_obs23523.fits")
-    obs2 = SpectrumDatasetOnOff.from_ogip_files(path + "pha_obs23592.fits")
+    obs1 = SpectrumDatasetOnOff.read(path + "pha_obs23523.fits")
+    obs2 = SpectrumDatasetOnOff.read(path + "pha_obs23592.fits")
     return [obs1, obs2]
 
 
@@ -767,9 +839,13 @@ def make_observation_list():
     mask_safe = RegionNDMap.from_geom(geom, dtype=bool)
     mask_safe.data += True
 
+    acceptance = RegionNDMap.from_geom(geom=geom, data=1)
+    acceptance_off_1 = RegionNDMap.from_geom(geom=geom, data=2)
+    acceptance_off_2 = RegionNDMap.from_geom(geom=geom, data=4)
+
     aeff = RegionNDMap.from_geom(geom_true, data=1, unit="m2")
     edisp = EDispKernelMap.from_gauss(
-        energy_axis=axis, energy_axis_true=axis, sigma=0.2, bias=0, geom=geom
+        energy_axis=axis, energy_axis_true=axis_true, sigma=0.2, bias=0, geom=geom
     )
 
     time_ref = Time("2010-01-01")
@@ -785,8 +861,8 @@ def make_observation_list():
         exposure=exposure,
         edisp=edisp,
         mask_safe=mask_safe,
-        acceptance=1,
-        acceptance_off=2,
+        acceptance=acceptance.copy(),
+        acceptance_off=acceptance_off_1,
         name="1",
         gti=gti1,
     )
@@ -796,8 +872,8 @@ def make_observation_list():
         exposure=exposure.copy(),
         edisp=edisp,
         mask_safe=mask_safe,
-        acceptance=1,
-        acceptance_off=4,
+        acceptance=acceptance.copy(),
+        acceptance_off=acceptance_off_2,
         name="2",
         gti=gti2,
     )
@@ -813,13 +889,14 @@ class TestSpectrumDatasetOnOffStack:
         # Change threshold to make stuff more interesting
 
         geom = self.datasets[0]._geom
-        data = geom.energy_mask(energy_min=1.2 * u.TeV, energy_max=50 * u.TeV)
-        self.datasets[0].mask_safe = RegionNDMap.from_geom(geom=geom, data=data)
+        self.datasets[0].mask_safe = geom.energy_mask(
+            energy_min=1.2 * u.TeV, energy_max=50 * u.TeV
+        )
 
-        data = geom.energy_mask(energy_max=20 * u.TeV)
-        self.datasets[1].mask_safe.data &= data
+        mask = geom.energy_mask(energy_max=20 * u.TeV)
+        self.datasets[1].mask_safe &= mask
 
-        self.stacked_dataset = self.datasets[0].copy()
+        self.stacked_dataset = self.datasets[0].to_masked()
         self.stacked_dataset.stack(self.datasets[1])
 
     def test_basic(self):
@@ -843,10 +920,10 @@ class TestSpectrumDatasetOnOffStack:
         energy_min, energy_max = self.stacked_dataset.energy_range
 
         assert energy_min.unit == "keV"
-        assert_allclose(energy_min.value, 8.912509e08, rtol=1e-3)
+        assert_allclose(energy_min, 8.912509e08, rtol=1e-3)
 
         assert energy_max.unit == "keV"
-        assert_allclose(energy_max.value, 4.466836e10, rtol=1e-3)
+        assert_allclose(energy_max, 4.466836e10, rtol=1e-3)
 
     def test_verify_npred(self):
         """Verifying npred is preserved during the stacking"""
@@ -871,7 +948,7 @@ class TestSpectrumDatasetOnOffStack:
         assert_allclose(npred_stacked, npred_summed, rtol=1e-6)
 
     def test_stack_backscal(self):
-        """Verify backscal stacking """
+        """Verify backscal stacking"""
         obs1, obs2 = make_observation_list()
         obs1.stack(obs2)
         assert_allclose(obs1.alpha.data[0], 1.25 / 4.0)
@@ -894,7 +971,7 @@ def test_datasets_stack_reduce():
 
     for obs_id in obs_ids:
         filename = f"$GAMMAPY_DATA/joint-crab/spectra/hess/pha_obs{obs_id}.fits"
-        ds = SpectrumDatasetOnOff.from_ogip_files(filename)
+        ds = SpectrumDatasetOnOff.read(filename)
         datasets.append(ds)
 
     stacked = datasets.stack_reduce(name="stacked")
@@ -909,16 +986,45 @@ def test_datasets_stack_reduce():
     assert stacked.name == "stacked"
 
 
+@requires_data("gammapy-data")
+def test_datasets_stack_reduce_no_off():
+    datasets = Datasets()
+    obs_ids = [23523, 23526, 23559, 23592]
+
+    for obs_id in obs_ids:
+        filename = f"$GAMMAPY_DATA/joint-crab/spectra/hess/pha_obs{obs_id}.fits"
+        ds = SpectrumDatasetOnOff.read(filename)
+        datasets.append(ds)
+
+    datasets[-1].counts_off = None
+
+    with pytest.raises(ValueError):
+        stacked = datasets.stack_reduce(name="stacked")
+
+    datasets[-1].mask_safe.data[...] = False
+    stacked = datasets.stack_reduce(name="stacked")
+    assert_allclose(stacked.exposure.meta["livetime"].to_value("s"), 4732.5469999)
+    assert stacked.counts == 369
+
+    datasets[0].mask_safe.data[...] = False
+
+    stacked = datasets.stack_reduce(name="stacked")
+    assert_allclose(stacked.exposure.meta["livetime"].to_value("s"), 3150.81024152)
+    assert stacked.counts == 245
+
+
 @requires_data("gammapy-data")
 def test_stack_livetime():
-    dataset_ref = SpectrumDatasetOnOff.from_ogip_files(
+    dataset_ref = SpectrumDatasetOnOff.read(
         "$GAMMAPY_DATA/joint-crab/spectra/hess/pha_obs23523.fits"
     )
 
     energy_axis = dataset_ref.counts.geom.axes["energy"]
     energy_axis_true = dataset_ref.exposure.geom.axes["energy_true"]
 
-    dataset = SpectrumDatasetOnOff.create(e_reco=energy_axis, e_true=energy_axis_true)
+    geom = RegionGeom(region=None, axes=[energy_axis])
+
+    dataset = SpectrumDatasetOnOff.create(geom=geom, energy_axis_true=energy_axis_true)
 
     dataset.stack(dataset_ref)
     assert_allclose(dataset.exposure.meta["livetime"], 1581.736758 * u.s)
@@ -969,11 +1075,7 @@ class TestFit:
         geom = RegionGeom(region=None, axes=[axis])
 
         self.src = RegionNDMap.from_geom(geom=geom, data=source_counts)
-
-        self.livetime = 1 * u.s
-        self.aeff = EffectiveAreaTable.from_constant(energy, "1 cm2").to_region_map(
-            region=None
-        )
+        self.exposure = RegionNDMap.from_geom(geom.as_energy_true, data=1, unit="cm2 s")
 
         npred_bkg = bkg_model.integral(energy[:-1], energy[1:]).value
 
@@ -987,7 +1089,7 @@ class TestFit:
         dataset = SpectrumDataset(
             models=self.source_model,
             counts=self.src,
-            exposure=self.aeff * self.livetime,
+            exposure=self.exposure,
         )
 
         npred = dataset.npred().data
@@ -998,11 +1100,11 @@ class TestFit:
 
         self.source_model.parameters["index"].value = 1.12
 
-        fit = Fit([dataset])
-        result = fit.run()
+        fit = Fit()
+        result = fit.run(datasets=[dataset])
 
         # These values are check with sherpa fits, do not change
-        pars = result.parameters
+        pars = self.source_model.parameters
         assert_allclose(pars["index"].value, 1.995525, rtol=1e-3)
         assert_allclose(pars["amplitude"].value, 100245.9, rtol=1e-3)
 
@@ -1010,19 +1112,22 @@ class TestFit:
         """WStat with on source and background spectrum"""
         on_vector = self.src.copy()
         on_vector.data += self.bkg.data
+        acceptance = RegionNDMap.from_geom(self.src.geom, data=1)
+        acceptance_off = RegionNDMap.from_geom(self.bkg.geom, data=1 / self.alpha)
+
         dataset = SpectrumDatasetOnOff(
             counts=on_vector,
             counts_off=self.off,
-            exposure=self.aeff * self.livetime,
-            acceptance=1,
-            acceptance_off=1 / self.alpha,
+            exposure=self.exposure,
+            acceptance=acceptance,
+            acceptance_off=acceptance_off,
         )
         dataset.models = self.source_model
 
         self.source_model.parameters.index = 1.12
 
-        fit = Fit([dataset])
-        result = fit.run()
+        fit = Fit()
+        result = fit.run(datasets=[dataset])
         pars = self.source_model.parameters
 
         assert_allclose(pars["index"].value, 1.997342, rtol=1e-3)
@@ -1040,8 +1145,8 @@ class TestFit:
         assert np.sum(dataset.mask_safe) == self.nbins
         energy_min, energy_max = dataset.energy_range
 
-        assert_allclose(energy_max.value, 10)
-        assert_allclose(energy_min.value, 0.1)
+        assert_allclose(energy_max, 10)
+        assert_allclose(energy_min, 0.1)
 
     def test_stat_profile(self):
         geom = self.src.geom
@@ -1050,14 +1155,17 @@ class TestFit:
 
         dataset = SpectrumDataset(
             models=self.source_model,
-            exposure=self.aeff * self.livetime,
+            exposure=self.exposure,
             counts=self.src,
             mask_safe=mask_safe,
         )
-        fit = Fit([dataset])
-        result = fit.run()
-        true_idx = result.parameters["index"].value
+        fit = Fit()
+        result = fit.run(datasets=[dataset])
+        true_idx = self.source_model.parameters["index"].value
+
         values = np.linspace(0.95 * true_idx, 1.05 * true_idx, 100)
-        profile = fit.stat_profile("index", values=values)
+        self.source_model.spectral_model.index.scan_values = values
+
+        profile = fit.stat_profile(datasets=[dataset], parameter="index")
         actual = values[np.argmin(profile["stat_scan"])]
         assert_allclose(actual, true_idx, rtol=0.01)
