@@ -2,7 +2,7 @@
 import numpy as np
 from numpy.testing import assert_allclose
 from gammapy.estimators.utils import find_peaks
-from gammapy.maps import Map
+from gammapy.maps import Map, MapAxis
 
 
 class TestFindPeaks:
@@ -44,3 +44,22 @@ class TestFindPeaks:
 
         table = find_peaks(image, threshold=3)
         assert len(table) == 0
+
+    def test_flat_map(self):
+        """Test a simple example"""
+        axis1 = MapAxis.from_edges([1, 2], name="axis1")
+        axis2 = MapAxis.from_edges([9, 10], name="axis2")
+        image = Map.create(npix=(10, 5), unit="s", axes=[axis1, axis2])
+        image.data[..., 3, 3] = 11
+        image.data[..., 3, 4] = 10
+        image.data[..., 3, 5] = 12
+        image.data[..., 3, 6] = np.nan
+        image.data[..., 0, 9] = 1e20
+
+        table = find_peaks(image, threshold=3)
+        row = table[0]
+
+        assert len(table) == 3
+        assert_allclose(row["value"], 1e20)
+        assert_allclose(row["ra"], 359.55)
+        assert_allclose(row["dec"], -0.2)
