@@ -17,11 +17,10 @@ from gammapy.makers import (
     ReflectedRegionsFinder,
     SpectrumDatasetMaker,
 )
-from gammapy.maps import MapAxis, WcsGeom, WcsNDMap
-from gammapy.utils.regions import compound_region_to_list
+from gammapy.maps import MapAxis, RegionGeom, WcsGeom
+from gammapy.utils.regions import compound_region_to_regions
 from gammapy.utils.testing import (
     assert_quantity_allclose,
-    mpl_plot_check,
     requires_data,
     requires_dependency,
 )
@@ -33,8 +32,7 @@ def exclusion_mask():
     pos = SkyCoord(83.63, 22.01, unit="deg", frame="icrs")
     exclusion_region = CircleSkyRegion(pos, Angle(0.3, "deg"))
     geom = WcsGeom.create(skydir=pos, binsz=0.02, width=10.0)
-    mask = geom.region_mask([exclusion_region], inside=False)
-    return WcsNDMap(geom, data=mask)
+    return ~geom.region_mask([exclusion_region])
 
 
 @pytest.fixture(scope="session")
@@ -80,28 +78,24 @@ def test_find_reflected_regions(
         exclusion_mask=exclusion_mask,
         min_distance_input="0 deg",
     )
-    finder.run()
-    regions = finder.reflected_regions
+    regions = finder.run()
     assert len(regions) == nreg1
     assert_quantity_allclose(regions[3].center.icrs.ra, reg3_ra, rtol=1e-2)
 
     # Test without exclusion
     finder.exclusion_mask = None
-    finder.run()
-    regions = finder.reflected_regions
+    regions = finder.run()
     assert len(regions) == nreg2
 
     # Test with too small exclusion
     small_mask = exclusion_mask.cutout(pointing, Angle("0.1 deg"))
     finder.exclusion_mask = small_mask
-    finder.run()
-    regions = finder.reflected_regions
+    regions = finder.run()
     assert len(regions) == nreg3
 
     # Test with maximum number of regions
     finder.max_region_number = 5
-    finder.run()
-    regions = finder.reflected_regions
+    regions = finder.run()
     assert len(regions) == 5
 
     # Test with an other type of region
@@ -115,8 +109,7 @@ def test_find_reflected_regions(
     )
     finder.region = on_ellipse_annulus
     finder.reference_map = None
-    finder.run()
-    regions = finder.reflected_regions
+    regions = finder.run()
     assert len(regions) == 5
 
 
@@ -124,10 +117,10 @@ center = SkyCoord(0.5, 0.0, unit="deg")
 other_region_finder_param = [
     (RectangleSkyRegion(center, 0.5 * u.deg, 0.5 * u.deg, angle=0 * u.deg), 3),
     (RectangleSkyRegion(center, 0.5 * u.deg, 1 * u.deg, angle=0 * u.deg), 1),
-    (RectangleSkyRegion(center, 0.5 * u.deg, 1 * u.deg, angle=90 * u.deg), 0),
+    (RectangleSkyRegion(center, 0.5 * u.deg, 1 * u.deg, angle=90 * u.deg), 1),
     (EllipseSkyRegion(center, 0.1 * u.deg, 1 * u.deg, angle=0 * u.deg), 2),
     (EllipseSkyRegion(center, 0.1 * u.deg, 1 * u.deg, angle=60 * u.deg), 3),
-    (EllipseSkyRegion(center, 0.1 * u.deg, 1 * u.deg, angle=90 * u.deg), 0),
+    (EllipseSkyRegion(center, 0.1 * u.deg, 1 * u.deg, angle=90 * u.deg), 2),
 ]
 
 
@@ -138,8 +131,7 @@ def test_non_circular_regions(region, nreg):
     finder = ReflectedRegionsFinder(
         center=pointing, region=region, min_distance_input="0 deg"
     )
-    finder.run()
-    regions = finder.reflected_regions
+    regions = finder.run()
     assert len(regions) == nreg
 
 
@@ -152,14 +144,9 @@ def test_bad_on_region(exclusion_mask, on_region):
         exclusion_mask=exclusion_mask,
         min_distance_input="0 deg",
     )
-    finder.run()
-    regions = finder.reflected_regions
+    regions = finder.run()
     assert len(regions) == 0
 
-    # try plotting
-    with mpl_plot_check():
-        finder.plot()
-
 
 @requires_data()
 def test_reflected_bkg_maker(on_region, reflected_bkg_maker, observations):
@@ -168,9 +155,8 @@ def test_reflected_bkg_maker(on_region, reflected_bkg_maker, observations):
     e_reco = MapAxis.from_edges(np.logspace(0, 2, 5) * u.TeV, name="energy")
     e_true = MapAxis.from_edges(np.logspace(-0.5, 2, 11) * u.TeV, name="energy_true")
 
-    dataset_empty = SpectrumDataset.create(
-        e_reco=e_reco, e_true=e_true, region=on_region
-    )
+    geom = RegionGeom(region=on_region, axes=[e_reco])
+    dataset_empty = SpectrumDataset.create(geom=geom, energy_axis_true=e_true)
 
     maker = SpectrumDatasetMaker(selection=["counts"])
 
@@ -182,14 +168,14 @@ def test_reflected_bkg_maker(on_region, reflected_bkg_maker, observations):
     assert_allclose(datasets[0].counts_off.data.sum(), 76)
     assert_allclose(datasets[1].counts_off.data.sum(), 60)
 
-    regions_0 = compound_region_to_list(datasets[0].counts_off.geom.region)
-    regions_1 = compound_region_to_list(datasets[1].counts_off.geom.region)
+    regions_0 = compound_region_to_regions(datasets[0].counts_off.geom.region)
+    regions_1 = compound_region_to_regions(datasets[1].counts_off.geom.region)
     assert_allclose(len(regions_0), 11)
     assert_allclose(len(regions_1), 11)
 
 
 @requires_data()
-def test_reflected_bkg_maker_no_off(reflected_bkg_maker, observations):
+def test_reflected_bkg_maker_no_off(reflected_bkg_maker, observations, caplog):
     pos = SkyCoord(83.6333313, 21.51444435, unit="deg", frame="icrs")
     radius = Angle(0.11, "deg")
     region = CircleSkyRegion(pos, radius)
@@ -200,7 +186,8 @@ def test_reflected_bkg_maker_no_off(reflected_bkg_maker, observations):
 
     e_reco = MapAxis.from_edges(np.logspace(0, 2, 5) * u.TeV, name="energy")
     e_true = MapAxis.from_edges(np.logspace(-0.5, 2, 11) * u.TeV, name="energy_true")
-    dataset_empty = SpectrumDataset.create(e_reco=e_reco, e_true=e_true, region=region)
+    geom = RegionGeom.create(region=region, axes=[e_reco])
+    dataset_empty = SpectrumDataset.create(geom=geom, energy_axis_true=e_true)
 
     for obs in observations:
         dataset = maker.run(dataset_empty, obs)
@@ -209,3 +196,42 @@ def test_reflected_bkg_maker_no_off(reflected_bkg_maker, observations):
 
     assert datasets[0].counts_off is None
     assert_allclose(datasets[0].acceptance_off, 0)
+    assert_allclose(datasets[0].mask_safe.data, False)
+
+    assert "WARNING" in [record.levelname for record in caplog.records]
+
+    message1 = (
+        f"ReflectedRegionsBackgroundMaker failed. "
+        f"No OFF region found outside exclusion mask for {datasets[0].name}."
+    )
+    message2 = (
+        f"ReflectedRegionsBackgroundMaker failed. "
+        f"Setting {datasets[0].name} mask to False."
+    )
+
+    assert message1 in [record.message for record in caplog.records]
+    assert message2 in [record.message for record in caplog.records]
+
+
+@requires_data()
+def test_reflected_bkg_maker_no_off_background(reflected_bkg_maker, observations):
+    pos = SkyCoord(83.6333313, 21.51444435, unit="deg", frame="icrs")
+    radius = Angle(0.11, "deg")
+    region = CircleSkyRegion(pos, radius)
+
+    maker = SpectrumDatasetMaker(selection=["counts", "background"])
+
+    datasets = []
+
+    e_reco = MapAxis.from_edges(np.logspace(0, 2, 5) * u.TeV, name="energy")
+    e_true = MapAxis.from_edges(np.logspace(-0.5, 2, 11) * u.TeV, name="energy_true")
+    geom = RegionGeom.create(region=region, axes=[e_reco])
+    dataset_empty = SpectrumDataset.create(geom=geom, energy_axis_true=e_true)
+
+    for obs in observations:
+        dataset = maker.run(dataset_empty, obs)
+        dataset_on_off = reflected_bkg_maker.run(dataset, obs)
+        datasets.append(dataset_on_off)
+
+    assert_allclose(datasets[0].counts_off.data, 0)
+    assert_allclose(datasets[0].acceptance_off, 0)
