@@ -1,10 +1,11 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 import logging
 import numpy as np
+from astropy import units as u
 from astropy.coordinates import Angle
-from regions import PointSkyRegion
-from gammapy.irf import EDispKernelMap, EffectiveAreaTable
+from gammapy.irf import EDispKernelMap
 from gammapy.maps import Map
+from gammapy.modeling.models import TemplateSpectralModel
 from .core import Maker
 
 __all__ = ["SafeMaskMaker"]
@@ -33,6 +34,9 @@ class SafeMaskMaker(Maker):
     position : `~astropy.coordinates.SkyCoord`
         Position at which the `aeff_percent` or `bias_percent` are computed. By default,
         it uses the position of the center of the map.
+    fixed_offset : `~astropy.coordinates.Angle`
+        offset, calculated from the pointing position, at which
+        the `aeff_percent` or `bias_percent` are computed.
     offset_max : str or `~astropy.units.Quantity`
         Maximum offset cut.
     """
@@ -52,6 +56,7 @@ class SafeMaskMaker(Maker):
         aeff_percent=10,
         bias_percent=10,
         position=None,
+        fixed_offset=None,
         offset_max="3 deg",
     ):
         methods = set(methods)
@@ -64,7 +69,12 @@ class SafeMaskMaker(Maker):
         self.aeff_percent = aeff_percent
         self.bias_percent = bias_percent
         self.position = position
+        self.fixed_offset = fixed_offset
         self.offset_max = Angle(offset_max)
+        if self.position and self.fixed_offset:
+            raise ValueError(
+                "`position` and `fixed_offset` attributes are mutually exclusive"
+            )
 
     def make_mask_offset_max(self, dataset, observation):
         """Make maximum offset mask.
@@ -81,6 +91,9 @@ class SafeMaskMaker(Maker):
         mask_safe : `~numpy.ndarray`
             Maximum offset mask.
         """
+        if observation is None:
+            raise ValueError("Method 'offset-max' requires an observation object.")
+
         separation = dataset._geom.separation(observation.pointing_radec)
         return separation < self.offset_max
 
@@ -100,55 +113,76 @@ class SafeMaskMaker(Maker):
         mask_safe : `~numpy.ndarray`
             Safe data range mask.
         """
+        if observation is None:
+            raise ValueError("Method 'offset-max' requires an observation object.")
+
         try:
-            energy_max = observation.aeff.high_threshold
-            energy_min = observation.aeff.low_threshold
+            energy_max = observation.aeff.meta["HI_THRES"] * u.TeV
+            energy_min = observation.aeff.meta["LO_THRES"] * u.TeV
         except KeyError:
-            log.warning(f"No thresholds defined for obs {observation.obs_id}")
+            log.warning(f"No default thresholds defined for obs {observation.obs_id}")
             energy_min, energy_max = None, None
 
-        return dataset.counts.geom.energy_mask(
-            energy_min=energy_min, energy_max=energy_max
-        )
+        return dataset._geom.energy_mask(energy_min=energy_min, energy_max=energy_max)
 
-    def make_mask_energy_aeff_max(self, dataset):
+    def make_mask_energy_aeff_max(self, dataset, observation=None):
         """Make safe energy mask from effective area maximum value.
 
         Parameters
         ----------
         dataset : `~gammapy.datasets.MapDataset` or `~gammapy.datasets.SpectrumDataset`
             Dataset to compute mask for.
+        observation: `~gammapy.data.Observation`
+            Observation to compute mask for. It is a mandatory argument when fixed_offset is set.
 
         Returns
         -------
         mask_safe : `~numpy.ndarray`
             Safe data range mask.
         """
-        geom = dataset._geom
+        geom, exposure = dataset._geom, dataset.exposure
 
-        if self.position is None:
-            position = PointSkyRegion(dataset.counts.geom.center_skydir)
+        if self.fixed_offset:
+            if observation:
+                position = observation.pointing_radec.directional_offset_by(
+                    position_angle=0.0 * u.deg, separation=self.fixed_offset
+                )
+            else:
+                raise ValueError(
+                    f"observation argument is mandatory with {self.fixed_offset}"
+                )
+
+        elif self.position:
+            position = self.position
         else:
-            position = PointSkyRegion(self.position)
+            position = geom.center_skydir
 
-        exposure = dataset.exposure.get_spectrum(position)
+        aeff = exposure.get_spectrum(position) / exposure.meta["livetime"]
+        model = TemplateSpectralModel.from_region_map(aeff)
 
-        energy = exposure.geom.axes["energy_true"]
-        aeff = EffectiveAreaTable(
-            energy_axis_true=energy,
-            data=(exposure.quantity / dataset.gti.time_sum).squeeze(),
+        energy_true = model.energy
+        energy_min = energy_true[np.where(model.values > 0)[0][0]]
+        energy_max = energy_true[-1]
+
+        aeff_thres = (self.aeff_percent / 100) * aeff.quantity.max()
+        inversion = model.inverse(
+            aeff_thres, energy_min=energy_min, energy_max=energy_max
         )
-        aeff_thres = (self.aeff_percent / 100) * aeff.max_area
-        energy_min = aeff.find_energy(aeff_thres)
+
+        if not np.isnan(inversion[0]):
+            energy_min = inversion[0]
+
         return geom.energy_mask(energy_min=energy_min)
 
-    def make_mask_energy_edisp_bias(self, dataset):
+    def make_mask_energy_edisp_bias(self, dataset, observation=None):
         """Make safe energy mask from energy dispersion bias.
 
         Parameters
         ----------
         dataset : `~gammapy.datasets.MapDataset` or `~gammapy.datasets.SpectrumDataset`
             Dataset to compute mask for.
+        observation: `~gammapy.data.Observation`
+            Observation to compute mask for. It is a mandatory argument when fixed_offset is set.
 
         Returns
         -------
@@ -156,18 +190,33 @@ class SafeMaskMaker(Maker):
             Safe data range mask.
         """
         edisp, geom = dataset.edisp, dataset._geom
+        position = None
+
+        if self.fixed_offset:
+            if observation:
+                position = observation.pointing_radec.directional_offset_by(
+                    position_angle=0 * u.deg, separation=self.fixed_offset
+                )
+            else:
+                raise ValueError(
+                    f"{observation} argument is mandatory with {self.fixed_offset}"
+                )
 
-        position = self.position
-        if position is None:
-            position = dataset.counts.geom.center_skydir
-            e_reco = dataset.counts.geom.axes["energy"].edges
         if isinstance(edisp, EDispKernelMap):
-            edisp = edisp.get_edisp_kernel(position)
+            if position:
+                edisp = edisp.get_edisp_kernel(position)
+            else:
+                edisp = edisp.get_edisp_kernel(self.position)
         else:
-            edisp = edisp.get_edisp_kernel(position, e_reco)
+            if position:
+                e_reco = dataset._geom.axes["energy"].edges
+                edisp = edisp.get_edisp_kernel(position, e_reco)
+            else:
+                e_reco = dataset._geom.axes["energy"].edges
+                edisp = edisp.get_edisp_kernel(self.position, e_reco)
 
         energy_min = edisp.get_bias_energy(self.bias_percent / 100)
-        return geom.energy_mask(energy_min=energy_min)
+        return geom.energy_mask(energy_min=energy_min[0])
 
     @staticmethod
     def make_mask_energy_bkg_peak(dataset):
@@ -187,13 +236,35 @@ class SafeMaskMaker(Maker):
         mask_safe : `~numpy.ndarray`
             Safe data range mask.
         """
-        geom = dataset.counts.geom
+        geom = dataset._geom
         background_spectrum = dataset.npred_background().get_spectrum()
         idx = np.argmax(background_spectrum.data, axis=0)
         energy_axis = geom.axes["energy"]
         energy_min = energy_axis.pix_to_coord(idx)
         return geom.energy_mask(energy_min=energy_min)
 
+    @staticmethod
+    def make_mask_bkg_invalid(dataset):
+        """Mask non-finite values and zeros values in background maps.
+
+        Parameters
+        ----------
+        dataset : `~gammapy.datasets.MapDataset` or `~gammapy.datasets.SpectrumDataset`
+            Dataset to compute mask for.
+
+        Returns
+        -------
+        mask_safe : `~numpy.ndarray`
+            Safe data range mask.
+        """
+        bkg = dataset.background.data
+        mask = np.isfinite(bkg)
+
+        if not dataset.stat_type == "wstat":
+            mask &= bkg > 0.0
+
+        return mask
+
     def run(self, dataset, observation=None):
         """Make safe data range mask.
 
@@ -211,6 +282,10 @@ class SafeMaskMaker(Maker):
         """
         mask_safe = np.ones(dataset._geom.data_shape, dtype=bool)
 
+        if dataset.background is not None:
+            # apply it first so only clipped values are removed for "bkg-peak"
+            mask_safe &= self.make_mask_bkg_invalid(dataset)
+
         if "offset-max" in self.methods:
             mask_safe &= self.make_mask_offset_max(dataset, observation)
 
@@ -226,5 +301,5 @@ class SafeMaskMaker(Maker):
         if "bkg-peak" in self.methods:
             mask_safe &= self.make_mask_energy_bkg_peak(dataset)
 
-        dataset.mask_safe = Map.from_geom(dataset._geom, data=mask_safe)
+        dataset.mask_safe = Map.from_geom(dataset._geom, data=mask_safe, dtype=bool)
         return dataset
