@@ -1,12 +1,10 @@
 # Licensed under a 3-clause BSD style license - see LICENSE.rst
 """Sky maps."""
-from .base import *
+from .axes import *
+from .coord import *
+from .core import *
 from .geom import *
 from .hpx import *
-from .hpxmap import *
-from .hpxnd import *
+from .maps import *
 from .region import *
-from .regionnd import *
 from .wcs import *
-from .wcsmap import *
-from .wcsnd import *
