@@ -18,6 +18,7 @@ SPATIAL_MODEL_REGISTRY = Registry(
         GeneralizedGaussianSpatialModel,
         PointSpatialModel,
         ShellSpatialModel,
+        Shell2SpatialModel,
     ]
 )
 """Registry of spatial model classes."""
@@ -51,14 +52,17 @@ SPECTRAL_MODEL_REGISTRY = Registry(
 TEMPORAL_MODEL_REGISTRY = Registry(
     [
         ConstantTemporalModel,
+        LinearTemporalModel,
         LightCurveTemplateTemporalModel,
         ExpDecayTemporalModel,
         GaussianTemporalModel,
+        PowerLawTemporalModel,
+        SineTemporalModel,
     ]
 )
 """Registry of temporal models classes."""
 
-MODEL_REGISTRY = Registry([SkyModel, FoVBackgroundModel, BackgroundModel])
+MODEL_REGISTRY = Registry([SkyModel, FoVBackgroundModel, TemplateNPredModel])
 """Registry of model classes"""
 
 
@@ -69,6 +73,8 @@ __all__ = [
     "SPECTRAL_MODEL_REGISTRY",
     "Models",
     "SkyModel",
+    "FoVBackgroundModel",
+    "TemplateNPredModel",
     "create_crab_spectral_model",
     "create_cosmic_ray_spectral_model",
     "SpatialModel",
