@@ -3,12 +3,13 @@
 import copy
 import numpy as np
 import astropy.units as u
-from gammapy.maps import Map, MapAxis, RegionGeom, WcsGeom
+from astropy.nddata import NoOverlapError
+from gammapy.maps import Map, MapAxis, WcsGeom
 from gammapy.modeling import Covariance, Parameters
 from gammapy.modeling.parameter import _get_parameters_str
 from gammapy.utils.fits import LazyFitsData
 from gammapy.utils.scripts import make_name, make_path
-from .core import Model, Models
+from .core import Model, ModelBase, Models
 from .spatial import ConstantSpatialModel, SpatialModel
 from .spectral import PowerLawNormSpectralModel, SpectralModel, TemplateSpectralModel
 from .temporal import TemporalModel
@@ -16,12 +17,12 @@ from .temporal import TemporalModel
 __all__ = [
     "SkyModel",
     "FoVBackgroundModel",
-    "BackgroundModel",
+    "TemplateNPredModel",
     "create_fermi_isotropic_diffuse_model",
 ]
 
 
-class SkyModel(Model):
+class SkyModel(ModelBase):
     """Sky model component.
 
     This model represents a factorised sky model.
@@ -184,11 +185,32 @@ class SkyModel(Model):
         """`~astropy.coordinates.SkyCoord`"""
         return getattr(self.spatial_model, "position", None)
 
+    @property
+    def position_lonlat(self):
+        """Spatial model center position `(lon, lat)` in rad and frame of the model"""
+        return getattr(self.spatial_model, "position_lonlat", None)
+
+    @property
+    def evaluation_bin_size_min(self):
+        """Minimal spatial bin size for spatial model evaluation."""
+        if (
+            self.spatial_model is not None
+            and self.spatial_model.evaluation_bin_size_min is not None
+        ):
+            return self.spatial_model.evaluation_bin_size_min
+        else:
+            return None
+
     @property
     def evaluation_radius(self):
         """`~astropy.coordinates.Angle`"""
         return self.spatial_model.evaluation_radius
 
+    @property
+    def evaluation_region(self):
+        """`~astropy.coordinates.Angle`"""
+        return self.spatial_model.evaluation_region
+
     @property
     def frame(self):
         return self.spatial_model.frame
@@ -196,7 +218,7 @@ class SkyModel(Model):
     def __add__(self, other):
         if isinstance(other, (Models, list)):
             return Models([self, *other])
-        elif isinstance(other, (SkyModel, BackgroundModel)):
+        elif isinstance(other, (SkyModel, TemplateNPredModel)):
             return Models([self, other])
         else:
             raise TypeError(f"Invalid type: {other!r}")
@@ -215,6 +237,45 @@ class SkyModel(Model):
             f"temporal_model={self.temporal_model!r})"
         )
 
+    def contributes(self, mask, margin="0 deg"):
+        """Check if a skymodel contributes within a mask map.
+
+        Parameters
+        ----------
+        mask : `~gammapy.maps.WcsNDMap` of boolean type
+            Map containing a boolean mask
+        margin : `~astropy.units.Quantity`
+            Add a margin in degree to the source evaluation radius.
+            Used to take into account PSF width.
+
+
+        Returns
+        -------
+        models : `DatasetModels`
+            Selected models contributing inside the region where mask==True
+        """
+        from gammapy.datasets.evaluator import CUTOUT_MARGIN
+
+        margin = u.Quantity(margin)
+
+        if not mask.geom.is_image:
+            mask = mask.reduce_over_axes(func=np.logical_or)
+
+        if mask.geom.is_region and mask.geom.region is not None:
+            geom = mask.geom.to_wcs_geom()
+            mask = geom.region_mask([mask.geom.region])
+
+        try:
+            mask_cutout = mask.cutout(
+                position=self.position,
+                width=(2 * self.evaluation_radius + CUTOUT_MARGIN + margin),
+            )
+            contributes = np.any(mask_cutout.data)
+        except (NoOverlapError, ValueError):
+            contributes = False
+
+        return contributes
+
     def evaluate(self, lon, lat, energy, time=None):
         """Evaluate the model at given points.
 
@@ -230,7 +291,7 @@ class SkyModel(Model):
         energy : `~astropy.units.Quantity`
             Energy coordinate
         time: `~astropy.time.Time`
-            Time co-ordinate
+            Time coordinate
 
         Returns
         -------
@@ -255,8 +316,8 @@ class SkyModel(Model):
 
     def evaluate_geom(self, geom, gti=None):
         """Evaluate model on `~gammapy.maps.Geom`."""
-        energy = geom.axes["energy_true"].center[:, np.newaxis, np.newaxis]
-        value = self.spectral_model(energy)
+        coords = geom.get_coord(sparse=True)
+        value = self.spectral_model(coords["energy_true"])
 
         if self.spatial_model:
             value = value * self.spatial_model.evaluate_geom(geom)
@@ -267,15 +328,21 @@ class SkyModel(Model):
 
         return value
 
-    def integrate_geom(self, geom, gti=None):
+    def integrate_geom(self, geom, gti=None, oversampling_factor=None):
         """Integrate model on `~gammapy.maps.Geom`.
 
+        See `~gammapy.modeling.models.SpatialModel.integrate_geom` and
+        `~gammapy.modeling.models.SpectralModel.integral`.
+
         Parameters
         ----------
-        geom : `Geom`
+        geom : `Geom` or `~gammapy.maps.RegionGeom`
             Map geometry
         gti : `GTI`
             GIT table
+        oversampling_factor : int or None
+            The oversampling factor to use for spatial integration.
+            Default is None: the factor is estimated from the model minimal bin size
 
         Returns
         -------
@@ -283,14 +350,18 @@ class SkyModel(Model):
             Predicted flux map
         """
         energy = geom.axes["energy_true"].edges
-        value = self.spectral_model.integral(energy[:-1], energy[1:],).reshape(
-            (-1, 1, 1)
-        )
+        value = self.spectral_model.integral(
+            energy[:-1],
+            energy[1:],
+        ).reshape((-1, 1, 1))
 
-        if self.spatial_model and not isinstance(geom, RegionGeom):
-            # TODO: integrate spatial model over region to correct for
-            #  containment
-            value = value * self.spatial_model.integrate_geom(geom).quantity
+        if self.spatial_model:
+            value = (
+                value
+                * self.spatial_model.integrate_geom(
+                    geom, oversampling_factor=oversampling_factor
+                ).quantity
+            )
 
         if self.temporal_model:
             integral = self.temporal_model.integral(gti.time_start, gti.time_stop)
@@ -324,13 +395,6 @@ class SkyModel(Model):
         data = {}
         data["name"] = self.name
         data["type"] = self.tag[0]
-        data["spectral"] = self.spectral_model.to_dict(full_output)
-
-        if self.spatial_model is not None:
-            data["spatial"] = self.spatial_model.to_dict(full_output)
-
-        if self.temporal_model is not None:
-            data["temporal"] = self.temporal_model.to_dict(full_output)
 
         if self.apply_irf != self._apply_irf_default:
             data["apply_irf"] = self.apply_irf
@@ -338,6 +402,14 @@ class SkyModel(Model):
         if self.datasets_names is not None:
             data["datasets_names"] = self.datasets_names
 
+        data.update(self.spectral_model.to_dict(full_output))
+
+        if self.spatial_model is not None:
+            data.update(self.spatial_model.to_dict(full_output))
+
+        if self.temporal_model is not None:
+            data.update(self.temporal_model.to_dict(full_output))
+
         return data
 
     @classmethod
@@ -350,13 +422,13 @@ class SkyModel(Model):
         )
 
         model_class = SPECTRAL_MODEL_REGISTRY.get_cls(data["spectral"]["type"])
-        spectral_model = model_class.from_dict(data["spectral"])
+        spectral_model = model_class.from_dict({"spectral": data["spectral"]})
 
         spatial_data = data.get("spatial")
 
         if spatial_data is not None:
             model_class = SPATIAL_MODEL_REGISTRY.get_cls(spatial_data["type"])
-            spatial_model = model_class.from_dict(spatial_data)
+            spatial_model = model_class.from_dict({"spatial": spatial_data})
         else:
             spatial_model = None
 
@@ -364,7 +436,7 @@ class SkyModel(Model):
 
         if temporal_data is not None:
             model_class = TEMPORAL_MODEL_REGISTRY.get_cls(temporal_data["type"])
-            temporal_model = model_class.from_dict(temporal_data)
+            temporal_model = model_class.from_dict({"temporal": temporal_data})
         else:
             temporal_model = None
 
@@ -443,8 +515,41 @@ class SkyModel(Model):
             **kwargs,
         )
 
+    def freeze(self, model_type=None):
+        """Freeze parameters depending on model type
 
-class FoVBackgroundModel(Model):
+        Parameters
+        ----------
+        model_type : {None, "spatial", "spectral", "temporal"}
+           freeze all parameters or only or only spatial/spectral/temporal.
+           Default is None so all parameters are frozen.
+        """
+        if model_type is None:
+            self.parameters.freeze_all()
+        else:
+            model = getattr(self, f"{model_type}_model")
+            model.freeze()
+
+    def unfreeze(self, model_type=None):
+        """Restore parameters frozen status to default depending on model type
+
+        Parameters
+        ----------
+        model_type : {None, "spatial", "spectral", "temporal"}
+           restore frozen status to default for all parameters or only spatial/spectral/temporal
+           Default is None so all parameters are restore to default frozen status.
+
+        """
+        if model_type is None:
+            for model_type in ["spectral", "spatial", "temporal"]:
+                self.unfreeze(model_type)
+        else:
+            model = getattr(self, f"{model_type}_model")
+            if model:
+                model.unfreeze()
+
+
+class FoVBackgroundModel(ModelBase):
     """Field of view background model
 
     The background model holds the correction parameters applied to
@@ -477,6 +582,10 @@ class FoVBackgroundModel(Model):
         self._spectral_model = spectral_model
         super().__init__()
 
+    def contributes(self, *args, **kwargs):
+        """FoV background models always contribute"""
+        return True
+
     @property
     def spectral_model(self):
         """Spectral norm model"""
@@ -512,18 +621,29 @@ class FoVBackgroundModel(Model):
 
     def evaluate_geom(self, geom):
         """Evaluate map"""
-        energy = geom.axes["energy"].center[:, np.newaxis, np.newaxis]
-        return self.evaluate(energy=energy)
+        coords = geom.get_coord(sparse=True)
+        return self.evaluate(energy=coords["energy"])
 
     def evaluate(self, energy):
         """Evaluate model"""
         return self.spectral_model(energy)
 
+    def copy(self, **kwargs):
+        """Copy SkyModel"""
+        kwargs.pop("name")
+        if "spectral_model" not in kwargs:
+            kwargs.setdefault("spectral_model", self.spectral_model.copy())
+        if "dataset_name" not in kwargs:
+            kwargs.setdefault("dataset_name", self.datasets_names[0])
+        return self.__class__(**kwargs)
+
     def to_dict(self, full_output=False):
         data = {}
         data["type"] = self.tag[0]
         data["datasets_names"] = self.datasets_names
-        data["spectral"] = self.spectral_model.to_dict(full_output=full_output)
+        data["spectral"] = self.spectral_model.to_dict(full_output=full_output)[
+            "spectral"
+        ]
         return data
 
     @classmethod
@@ -552,10 +672,28 @@ class FoVBackgroundModel(Model):
         if len(datasets_names) > 1:
             raise ValueError("FoVBackgroundModel can only be assigned to one dataset")
 
-        return cls(spectral_model=spectral_model, dataset_name=datasets_names[0],)
+        return cls(
+            spectral_model=spectral_model,
+            dataset_name=datasets_names[0],
+        )
+
+    def reset_to_default(self):
+        """Reset parameter values to default"""
+        values = self.spectral_model.default_parameters.value
+        self.spectral_model.parameters.value = values
+
+    def freeze(self, model_type="spectral"):
+        """Freeze model parameters"""
+        if model_type is None or model_type == "spectral":
+            self._spectral_model.freeze()
+
+    def unfreeze(self, model_type="spectral"):
+        """Restore parameters frozen status to default"""
+        if model_type is None or model_type == "spectral":
+            self._spectral_model.unfreeze()
 
 
-class BackgroundModel(Model):
+class TemplateNPredModel(ModelBase):
     """Background model.
 
     Create a new map by a tilt and normalization on the available map
@@ -569,11 +707,16 @@ class BackgroundModel(Model):
         default is `~gammapy.modeling.models.PowerLawNormSpectralModel`
     """
 
-    tag = "BackgroundModel"
+    tag = "TemplateNPredModel"
     map = LazyFitsData(cache=True)
 
     def __init__(
-        self, map, spectral_model=None, name=None, filename=None, datasets_names=None,
+        self,
+        map,
+        spectral_model=None,
+        name=None,
+        filename=None,
+        datasets_names=None,
     ):
         if isinstance(map, Map):
             axis = map.geom.axes["energy"]
@@ -592,6 +735,9 @@ class BackgroundModel(Model):
 
         self.spectral_model = spectral_model
 
+        if isinstance(datasets_names, str):
+            datasets_names = [datasets_names]
+
         if isinstance(datasets_names, list):
             if len(datasets_names) != 1:
                 raise ValueError(
@@ -644,7 +790,7 @@ class BackgroundModel(Model):
         data = {}
         data["name"] = self.name
         data["type"] = self.tag
-        data["spectral"] = self.spectral_model.to_dict(full_output)
+        data["spectral"] = self.spectral_model.to_dict(full_output)["spectral"]
 
         if self.filename is not None:
             data["filename"] = self.filename
@@ -709,7 +855,7 @@ class BackgroundModel(Model):
 
         Returns
         -------
-        cutout : `BackgroundModel`
+        cutout : `TemplateNPredModel`
             Cutout background model.
         """
         cutout_kwargs = {"position": position, "width": width, "mode": mode}
@@ -718,19 +864,23 @@ class BackgroundModel(Model):
         spectral_model = self.spectral_model.copy()
         return self.__class__(bkg_map, spectral_model=spectral_model, name=name)
 
-    def stack(self, other, weights=None):
+    def stack(self, other, weights=None, nan_to_num=True):
         """Stack background model in place.
 
         Stacking the background model resets the current parameters values.
 
         Parameters
         ----------
-        other : `BackgroundModel`
+        other : `TemplateNPredModel`
             Other background model.
+        nan_to_num: bool
+            Non-finite values are replaced by zero if True (default).
         """
         bkg = self.evaluate()
+        if nan_to_num:
+            bkg.data[~np.isfinite(bkg.data)] = 0
         other_bkg = other.evaluate()
-        bkg.stack(other_bkg, weights=weights)
+        bkg.stack(other_bkg, weights=weights, nan_to_num=nan_to_num)
         self.map = bkg
 
         # reset parameter values
@@ -760,6 +910,16 @@ class BackgroundModel(Model):
         """`~astropy.coordinates.Angle`"""
         return np.max(self.map.geom.width) / 2.0
 
+    def freeze(self, model_type="spectral"):
+        """Freeze model parameters"""
+        if model_type is None or model_type == "spectral":
+            self._spectral_model.freeze()
+
+    def unfreeze(self, model_type="spectral"):
+        """Restore parameters frozen status to default"""
+        if model_type is None or model_type == "spectral":
+            self._spectral_model.unfreeze()
+
 
 def create_fermi_isotropic_diffuse_model(filename, **kwargs):
     """Read Fermi isotropic diffuse model.
