@@ -7,16 +7,42 @@ import scipy.special
 import astropy.units as u
 from astropy import constants as const
 from astropy.table import Table
-from gammapy.maps import MapAxis
-from gammapy.maps.utils import edges_from_lo_hi
+from astropy.utils.decorators import classproperty
+from astropy.visualization import quantity_support
+from gammapy.maps import MapAxis, RegionNDMap
 from gammapy.modeling import Parameter, Parameters
 from gammapy.utils.integrate import trapz_loglog
 from gammapy.utils.interpolation import (
     ScaledRegularGridInterpolator,
     interpolation_scale,
 )
+from gammapy.utils.roots import find_roots
 from gammapy.utils.scripts import make_path
-from .core import Model
+from .core import ModelBase
+
+
+def scale_plot_flux(flux, energy_power=0):
+    """Scale flux to plot
+
+    Parameters
+    ----------
+    flux : `Map`
+        Flux map
+    energy_power : int, optional
+        Power of energy to multiply flux axis with
+
+    Returns
+    -------
+    flux : `Map`
+        Scaled flux map
+    """
+    energy = flux.geom.get_coord(sparse=True)["energy"]
+    try:
+        eunit = [_ for _ in flux.unit.bases if _.physical_type == "energy"][0]
+    except IndexError:
+        eunit = energy.unit
+    y = flux * np.power(energy, energy_power)
+    return y.to_unit(flux.unit * eunit ** energy_power)
 
 
 def integrate_spectrum(func, energy_min, energy_max, ndecade=100):
@@ -36,13 +62,14 @@ def integrate_spectrum(func, energy_min, energy_max, ndecade=100):
         Number of grid points per decade used for the integration.
         Default : 100
     """
-    num = np.max(ndecade * np.log10(energy_max / energy_min))
+    # Here we impose to duplicate the number
+    num = np.maximum(np.max(ndecade * np.log10(energy_max / energy_min)), 2)
     energy = np.geomspace(energy_min, energy_max, num=int(num), axis=-1)
     integral = trapz_loglog(func(energy), energy, axis=-1)
     return integral.sum(axis=0)
 
 
-class SpectralModel(Model):
+class SpectralModel(ModelBase):
     """Spectral model base class."""
 
     _type = "spectral"
@@ -52,14 +79,10 @@ class SpectralModel(Model):
         kwargs = self._convert_evaluate_unit(kwargs, energy)
         return self.evaluate(energy, **kwargs)
 
-    @property
-    def type(self):
-        return self._type
-
-    @property
-    def is_norm_spectral_model(self):
+    @classproperty
+    def is_norm_spectral_model(cls):
         """Whether model is a norm spectral model"""
-        return "Norm" in self.__class__.__name__
+        return "Norm" in cls.__name__
 
     @staticmethod
     def _convert_evaluate_unit(kwargs_ref, energy):
@@ -92,10 +115,28 @@ class SpectralModel(Model):
     def __rsub__(self, model):
         return self.__sub__(model)
 
-    def _evaluate_gradient(self, energy, eps):
-        n = len(self.parameters)
-        f = self(energy)
-        shape = (n, len(np.atleast_1d(energy)))
+    def _propagate_error(self, epsilon, fct, **kwargs):
+        """Evaluate error for a given function with uncertainty propagation.
+
+        Parameters
+        ----------
+        fct : `~astropy.units.Quantity`
+            Function to estimate the error.
+        epsilon : float
+            Step size of the gradient evaluation. Given as a
+            fraction of the parameter error.
+        **kwargs : dict
+            Keyword argument
+
+        Returns
+        -------
+        f_cov : `~astropy.units.Quantity`
+            Error of the given function.
+        """
+        eps = np.sqrt(np.diag(self.covariance)) * epsilon
+
+        n, f_0 = len(self.parameters), fct(**kwargs)
+        shape = (n, len(np.atleast_1d(f_0)))
         df_dp = np.zeros(shape)
 
         for idx, parameter in enumerate(self.parameters):
@@ -103,13 +144,14 @@ class SpectralModel(Model):
                 continue
 
             parameter.value += eps[idx]
-            df = self(energy) - f
-            df_dp[idx] = df.value / eps[idx]
+            df = fct(**kwargs) - f_0
 
-            # Reset model to original parameter
+            df_dp[idx] = df.value / eps[idx]
             parameter.value -= eps[idx]
 
-        return df_dp
+        f_cov = df_dp.T @ self.covariance @ df_dp
+        f_err = np.sqrt(np.diagonal(f_cov))
+        return u.Quantity([f_0.value, f_err], unit=f_0.unit)
 
     def evaluate_error(self, energy, epsilon=1e-4):
         """Evaluate spectral model with error propagation.
@@ -127,15 +169,7 @@ class SpectralModel(Model):
         dnde, dnde_error : tuple of `~astropy.units.Quantity`
             Tuple of flux and flux error.
         """
-        p_cov = self.covariance
-        eps = np.sqrt(np.diag(p_cov)) * epsilon
-
-        df_dp = self._evaluate_gradient(energy, eps)
-        f_cov = df_dp.T @ p_cov @ df_dp
-        f_err = np.sqrt(np.diagonal(f_cov))
-
-        q = self(energy)
-        return u.Quantity([q.value, f_err], unit=q.unit)
+        return self._propagate_error(epsilon=epsilon, fct=self, energy=energy)
 
     def integral(self, energy_min, energy_max, **kwargs):
         r"""Integrate spectral model numerically if no analytical solution defined.
@@ -158,7 +192,7 @@ class SpectralModel(Model):
         else:
             return integrate_spectrum(self, energy_min, energy_max, **kwargs)
 
-    def integral_error(self, energy_min, energy_max):
+    def integral_error(self, energy_min, energy_max, epsilon=1e-4, **kwargs):
         """Evaluate the error of the integral flux of a given spectrum in
         a given energy range.
 
@@ -166,55 +200,23 @@ class SpectralModel(Model):
         ----------
         energy_min, energy_max :  `~astropy.units.Quantity`
             Lower and upper bound of integration range.
-
-        Returns
-        -------
-        flux, flux_err : tuple of `~astropy.units.Quantity`
-            Integral flux and flux error betwen energy_min and energy_max.
-        """
-        energy = np.sqrt(energy_min * energy_max)
-        flux = self.integral(energy_min, energy_max)
-        dnde, dnde_err = self.evaluate_error(energy, epsilon=1e-4)
-        flux_err = flux * dnde_err / dnde
-        return u.Quantity([flux.value, flux_err.value], unit=flux.unit)
-
-    def _propagate_error(self, fct, energy_min, energy_max, eps):
-        """Evaluate error of a given function with uncertainty propagation.
-
-        Parameters
-        ----------
-        fct : `~astropy.units.Quantity`
-            Function to estimate the error.
-        energy_min, energy_max : `~astropy.units.Quantity`
-            Array of lower and upper bound of integration range.
         epsilon : float
             Step size of the gradient evaluation. Given as a
             fraction of the parameter error.
 
+
         Returns
         -------
-        f_cov : `~astropy.units.Quantity`
-            Error of the given function.
+        flux, flux_err : tuple of `~astropy.units.Quantity`
+            Integral flux and flux error between energy_min and energy_max.
         """
-        n = len(self.parameters)
-        C = self.covariance
-        f = fct
-        shape = (n, len(np.atleast_1d(energy_min)))
-        df_dp = np.zeros(shape)
-
-        for idx, parameter in enumerate(self.parameters):
-            if parameter.frozen or eps[idx] == 0:
-                continue
-
-            parameter.value += eps[idx]
-            df = self.energy_flux(energy_min, energy_max) - f
-            df_dp[idx] = df.value / eps[idx]
-
-            # Reset model to original parameter
-            parameter.value -= eps[idx]
-
-        f_cov = df_dp.T @ C @ df_dp
-        return np.sqrt(np.diagonal(f_cov))
+        return self._propagate_error(
+            epsilon=epsilon,
+            fct=self.integral,
+            energy_min=energy_min,
+            energy_max=energy_max,
+            **kwargs,
+        )
 
     def energy_flux(self, energy_min, energy_max, **kwargs):
         r"""Compute energy flux in given energy range.
@@ -248,24 +250,80 @@ class SpectralModel(Model):
         ----------
         energy_min, energy_max :  `~astropy.units.Quantity`
             Lower and upper bound of integration range.
+        epsilon : float
+            Step size of the gradient evaluation. Given as a
+            fraction of the parameter error.
+
 
         Returns
         -------
         energy_flux, energy_flux_err : tuple of `~astropy.units.Quantity`
-            Energy flux and energy flux error betwen energy_min and energy_max.
+            Energy flux and energy flux error between energy_min and energy_max.
         """
-        p_cov = self.covariance
-        eps = np.sqrt(np.diag(p_cov)) * epsilon
-        enrg_flux = self.energy_flux(energy_min, energy_max, **kwargs)
-        enrg_flux_err = self._propagate_error(enrg_flux, energy_min, energy_max, eps)
-        return u.Quantity([enrg_flux.value, enrg_flux_err], unit=enrg_flux.unit)
+        return self._propagate_error(
+            epsilon=epsilon,
+            fct=self.energy_flux,
+            energy_min=energy_min,
+            energy_max=energy_max,
+            **kwargs,
+        )
+
+    def reference_fluxes(self, energy_axis):
+        """Get reference fluxes for a given energy axis.
+
+        Parameters
+        ----------
+        energy_axis : `MapAxis`
+            Energy axis
+
+        Returns
+        -------
+        fluxes : dict of `~astropy.units.Quantity`
+            Reference fluxes
+        """
+        energy = energy_axis.center
+        energy_min, energy_max = energy_axis.edges_min, energy_axis.edges_max
+        return {
+            "e_ref": energy,
+            "e_min": energy_min,
+            "e_max": energy_max,
+            "ref_dnde": self(energy),
+            "ref_flux": self.integral(energy_min, energy_max),
+            "ref_eflux": self.energy_flux(energy_min, energy_max),
+            "ref_e2dnde": self(energy) * energy ** 2,
+        }
+
+    def _get_plot_flux(self, energy, sed_type):
+        flux = RegionNDMap.create(region=None, axes=[energy])
+        flux_err = RegionNDMap.create(region=None, axes=[energy])
+
+        if sed_type in ["dnde", "norm"]:
+            flux.quantity, flux_err.quantity = self.evaluate_error(energy.center)
+
+        elif sed_type == "e2dnde":
+            flux.quantity, flux_err.quantity = energy.center ** 2 * self.evaluate_error(
+                energy.center
+            )
+
+        elif sed_type == "flux":
+            flux.quantity, flux_err.quantity = self.integral_error(
+                energy.edges_min, energy.edges_max
+            )
+
+        elif sed_type == "eflux":
+            flux.quantity, flux_err.quantity = self.energy_flux_error(
+                energy.edges_min, energy.edges_max
+            )
+        else:
+            raise ValueError(f"Not a valid SED type: '{sed_type}'")
+
+        return flux, flux_err
 
     def plot(
         self,
-        energy_range,
+        energy_bounds,
         ax=None,
-        energy_unit="TeV",
-        flux_unit="cm-2 s-1 TeV-1",
+        sed_type="dnde",
         energy_power=0,
         n_points=100,
         **kwargs,
@@ -281,23 +339,23 @@ class SpectralModel(Model):
             from astropy import units as u
 
             pwl = ExpCutoffPowerLawSpectralModel()
-            ax = pwl.plot(energy_range=(0.1, 100) * u.TeV)
+            ax = pwl.plot(energy_bounds=(0.1, 100) * u.TeV)
             ax.set_yscale('linear')
 
         Parameters
         ----------
         ax : `~matplotlib.axes.Axes`, optional
             Axis
-        energy_range : `~astropy.units.Quantity`
-            Plot range
-        energy_unit : str, `~astropy.units.Unit`, optional
-            Unit of the energy axis
-        flux_unit : str, `~astropy.units.Unit`, optional
-            Unit of the flux axis
+        energy_bounds : `~astropy.units.Quantity`
+            Plot energy bounds passed to MapAxis.from_energy_bounds
+        sed_type : {"dnde", "flux", "eflux", "e2dnde"}
+            Evaluation methods of the model
         energy_power : int, optional
             Power of energy to multiply flux axis with
         n_points : int, optional
             Number of evaluation nodes
+        **kwargs : dict
+            Keyword arguments forwarded to `~matplotlib.pyplot.plot`
 
         Returns
         -------
@@ -305,30 +363,39 @@ class SpectralModel(Model):
             Axis
         """
         import matplotlib.pyplot as plt
+        from gammapy.estimators.map.core import DEFAULT_UNIT
 
         ax = plt.gca() if ax is None else ax
 
-        energy_min, energy_max = energy_range
+        if self.is_norm_spectral_model:
+            sed_type = "norm"
+
+        energy_min, energy_max = energy_bounds
         energy = MapAxis.from_energy_bounds(
-            energy_min, energy_max, n_points, energy_unit
-        ).edges
+            energy_min,
+            energy_max,
+            n_points,
+        )
 
-        # evaluate model
-        flux = self(energy).to(flux_unit)
+        kwargs.setdefault(
+            "yunits", DEFAULT_UNIT[sed_type] * energy.unit ** energy_power
+        )
 
-        y = self._plot_scale_flux(energy, flux, energy_power)
+        flux, _ = self._get_plot_flux(sed_type=sed_type, energy=energy)
 
-        ax.plot(energy.value, y.value, **kwargs)
+        flux = scale_plot_flux(flux, energy_power=energy_power)
 
-        self._plot_format_ax(ax, energy, y, energy_power)
+        with quantity_support():
+            ax.plot(energy.center, flux.quantity[:, 0, 0], **kwargs)
+
+        self._plot_format_ax(ax, energy_power, sed_type)
         return ax
 
     def plot_error(
         self,
-        energy_range,
+        energy_bounds,
         ax=None,
-        energy_unit="TeV",
-        flux_unit="cm-2 s-1 TeV-1",
+        sed_type="dnde",
         energy_power=0,
         n_points=100,
         **kwargs,
@@ -337,28 +404,26 @@ class SpectralModel(Model):
 
         .. note::
 
-            This method calls ``ax.set_yscale("log", nonposy='clip')`` and
+            This method calls ``ax.set_yscale("log", nonpositive='clip')`` and
             ``ax.set_xscale("log", nonposx='clip')`` to create a log-log representation.
             The additional argument ``nonposx='clip'`` avoids artefacts in the plot,
             when the error band extends to negative values (see also
             https://github.com/matplotlib/matplotlib/issues/8623).
 
-            When you call ``plt.loglog()`` or ``plt.semilogy()`` explicitely in your
+            When you call ``plt.loglog()`` or ``plt.semilogy()`` explicitly in your
             plotting code and the error band extends to negative values, it is not
             shown correctly. To circumvent this issue also use
-            ``plt.loglog(nonposx='clip', nonposy='clip')``
-            or ``plt.semilogy(nonposy='clip')``.
+            ``plt.loglog(nonposx='clip', nonpositive='clip')``
+            or ``plt.semilogy(nonpositive='clip')``.
 
         Parameters
         ----------
         ax : `~matplotlib.axes.Axes`, optional
             Axis
-        energy_range : `~astropy.units.Quantity`
-            Plot range
-        energy_unit : str, `~astropy.units.Unit`, optional
-            Unit of the energy axis
-        flux_unit : str, `~astropy.units.Unit`, optional
-            Unit of the flux axis
+        energy_bounds : `~astropy.units.Quantity`
+            Plot energy bounds passed to MapAxis.from_energy_bounds
+        sed_type : {"dnde", "flux", "eflux", "e2dnde"}
+            Evaluation methods of the model
         energy_power : int, optional
             Power of energy to multiply flux axis with
         n_points : int, optional
@@ -372,50 +437,47 @@ class SpectralModel(Model):
             Axis
         """
         import matplotlib.pyplot as plt
+        from gammapy.estimators.map.core import DEFAULT_UNIT
 
         ax = plt.gca() if ax is None else ax
 
+        if self.is_norm_spectral_model:
+            sed_type = "norm"
+
+        energy_min, energy_max = energy_bounds
+        energy = MapAxis.from_energy_bounds(
+            energy_min,
+            energy_max,
+            n_points,
+        )
+
         kwargs.setdefault("facecolor", "black")
         kwargs.setdefault("alpha", 0.2)
         kwargs.setdefault("linewidth", 0)
+        kwargs.setdefault(
+            "yunits", DEFAULT_UNIT[sed_type] * energy.unit ** energy_power
+        )
 
-        energy_min, energy_max = energy_range
-        energy = MapAxis.from_energy_bounds(
-            energy_min, energy_max, n_points, energy_unit
-        ).edges
+        flux, flux_err = self._get_plot_flux(sed_type=sed_type, energy=energy)
+        y_lo = scale_plot_flux(flux - flux_err, energy_power).quantity[:, 0, 0]
+        y_hi = scale_plot_flux(flux + flux_err, energy_power).quantity[:, 0, 0]
 
-        flux, flux_err = self.evaluate_error(energy).to(flux_unit)
+        with quantity_support():
+            ax.fill_between(energy.center, y_lo, y_hi, **kwargs)
 
-        y_lo = self._plot_scale_flux(energy, flux - flux_err, energy_power)
-        y_hi = self._plot_scale_flux(energy, flux + flux_err, energy_power)
-
-        where = (energy >= energy_range[0]) & (energy <= energy_range[1])
-        ax.fill_between(energy.value, y_lo.value, y_hi.value, where=where, **kwargs)
-
-        self._plot_format_ax(ax, energy, y_lo, energy_power)
+        self._plot_format_ax(ax, energy_power, sed_type)
         return ax
 
-    def _plot_format_ax(self, ax, energy, y, energy_power):
-        ax.set_xlabel(f"Energy [{energy.unit}]")
-        if energy_power > 0:
-            ax.set_ylabel(f"E{energy_power} * Flux [{y.unit}]")
-        else:
-            ax.set_ylabel(f"Flux [{y.unit}]")
-
-        ax.set_xscale("log", nonposx="clip")
-        ax.set_yscale("log", nonposy="clip")
-
-        if "norm" in self.__class__.__name__.lower():
-            ax.set_ylabel(f"Norm [A.U.]")
-
     @staticmethod
-    def _plot_scale_flux(energy, flux, energy_power):
-        try:
-            eunit = [_ for _ in flux.unit.bases if _.physical_type == "energy"][0]
-        except IndexError:
-            eunit = energy.unit
-        y = flux * np.power(energy, energy_power)
-        return y.to(flux.unit * eunit ** energy_power)
+    def _plot_format_ax(ax, energy_power, sed_type):
+        ax.set_xlabel(f"Energy [{ax.xaxis.units}]")
+        if energy_power > 0:
+            ax.set_ylabel(f"e{energy_power} * {sed_type} [{ax.yaxis.units}]")
+        else:
+            ax.set_ylabel(f"{sed_type} [{ax.yaxis.units}]")
+
+        ax.set_xscale("log", nonpositive="clip")
+        ax.set_yscale("log", nonpositive="clip")
 
     def spectral_index(self, energy, epsilon=1e-5):
         """Compute spectral index at given energy.
@@ -446,32 +508,54 @@ class SpectralModel(Model):
         value : `~astropy.units.Quantity`
             Function value of the spectral model.
         energy_min : `~astropy.units.Quantity`
-            Lower bracket value in case solution is not unique.
+            Lower energy bound of the roots finding
         energy_max : `~astropy.units.Quantity`
-            Upper bracket value in case solution is not unique.
+            Upper energy bound of the roots finding
 
         Returns
         -------
         energy : `~astropy.units.Quantity`
             Energies at which the model has the given ``value``.
         """
+
         eunit = "TeV"
+        energy_min = energy_min.to(eunit)
+        energy_max = energy_max.to(eunit)
 
+        def f(x):
+            # scale by 1e12 to achieve better precision
+            energy = u.Quantity(x, eunit, copy=False)
+            y = self(energy).to_value(value.unit)
+            return 1e12 * (y - value.value)
+
+        roots, res = find_roots(f, energy_min, energy_max, points_scale="log")
+        return roots
+
+    def inverse_all(self, values, energy_min=0.1 * u.TeV, energy_max=100 * u.TeV):
+        """Return energies for multiple function values of the spectral model.
+
+        Calls the `scipy.optimize.brentq` numerical root finding method.
+
+        Parameters
+        ----------
+        values : `~astropy.units.Quantity`
+            Function values of the spectral model.
+        energy_min : `~astropy.units.Quantity`
+            Lower energy bound of the roots finding
+        energy_max : `~astropy.units.Quantity`
+            Upper energy bound of the roots finding
+
+        Returns
+        -------
+        energy : list of `~astropy.units.Quantity`
+            each element contain the energies at which the model
+            has corresponding value of ``values``.
+        """
         energies = []
-        for val in np.atleast_1d(value):
-
-            def f(x):
-                # scale by 1e12 to achieve better precision
-                energy = u.Quantity(x, eunit, copy=False)
-                y = self(energy).to_value(value.unit)
-                return 1e12 * (y - val.value)
-
-            energy = scipy.optimize.brentq(
-                f, energy_min.to_value(eunit), energy_max.to_value(eunit)
-            )
-            energies.append(energy)
-
-        return u.Quantity(energies, eunit, copy=False)
+        for val in np.atleast_1d(values):
+            res = self.inverse(val, energy_min, energy_max)
+            energies.append(res)
+        return energies
 
 
 class ConstantSpectralModel(SpectralModel):
@@ -526,21 +610,26 @@ class CompoundSpectralModel(SpectralModel):
         return self.operator(val1, val2)
 
     def to_dict(self, full_output=False):
+        dict1 = self.model1.to_dict(full_output)
+        dict2 = self.model2.to_dict(full_output)
         return {
-            "type": self.tag[0],
-            "model1": self.model1.to_dict(full_output),
-            "model2": self.model2.to_dict(full_output),
-            "operator": self.operator.__name__,
+            self._type: {
+                "type": self.tag[0],
+                "model1": dict1["spectral"],  # for cleaner output
+                "model2": dict2["spectral"],
+                "operator": self.operator.__name__,
+            }
         }
 
     @classmethod
     def from_dict(cls, data):
         from gammapy.modeling.models import SPECTRAL_MODEL_REGISTRY
 
+        data = data["spectral"]
         model1_cls = SPECTRAL_MODEL_REGISTRY.get_cls(data["model1"]["type"])
-        model1 = model1_cls.from_dict(data["model1"])
+        model1 = model1_cls.from_dict({"spectral": data["model1"]})
         model2_cls = SPECTRAL_MODEL_REGISTRY.get_cls(data["model2"]["type"])
-        model2 = model2_cls.from_dict(data["model2"])
+        model2 = model2_cls.from_dict({"spectral": data["model2"]})
         op = getattr(operator, data["operator"])
         return cls(model1, model2, op)
 
@@ -566,7 +655,9 @@ class PowerLawSpectralModel(SpectralModel):
 
     tag = ["PowerLawSpectralModel", "pl"]
     index = Parameter("index", 2.0)
-    amplitude = Parameter("amplitude", "1e-12 cm-2 s-1 TeV-1")
+    amplitude = Parameter(
+        "amplitude", "1e-12 cm-2 s-1 TeV-1", scale_method="scale10", interp="log"
+    )
     reference = Parameter("reference", "1 TeV", frozen=True)
 
     @staticmethod
@@ -636,7 +727,7 @@ class PowerLawSpectralModel(SpectralModel):
 
         return energy_flux
 
-    def inverse(self, value):
+    def inverse(self, value, *args):
         """Return energy for a given function value of the spectral model.
 
         Parameters
@@ -682,7 +773,7 @@ class PowerLawNormSpectralModel(SpectralModel):
     """
 
     tag = ["PowerLawNormSpectralModel", "pl-norm"]
-    norm = Parameter("norm", 1, unit="")
+    norm = Parameter("norm", 1, unit="", interp="log")
     tilt = Parameter("tilt", 0, frozen=True)
     reference = Parameter("reference", "1 TeV", frozen=True)
 
@@ -729,7 +820,7 @@ class PowerLawNormSpectralModel(SpectralModel):
 
         return energy_flux
 
-    def inverse(self, value):
+    def inverse(self, value, *args):
         """Return energy for a given function value of the spectral model.
 
         Parameters
@@ -779,7 +870,9 @@ class PowerLaw2SpectralModel(SpectralModel):
     """
     tag = ["PowerLaw2SpectralModel", "pl-2"]
 
-    amplitude = Parameter("amplitude", "1e-12 cm-2 s-1")
+    amplitude = Parameter(
+        "amplitude", "1e-12 cm-2 s-1", scale_method="scale10", interp="log"
+    )
     index = Parameter("index", 2)
     emin = Parameter("emin", "0.1 TeV", frozen=True)
     emax = Parameter("emax", "100 TeV", frozen=True)
@@ -817,7 +910,7 @@ class PowerLaw2SpectralModel(SpectralModel):
 
         return amplitude * top / bottom
 
-    def inverse(self, value):
+    def inverse(self, value, *args):
         """Return energy for a given function value of the spectral model.
 
         Parameters
@@ -861,7 +954,9 @@ class BrokenPowerLawSpectralModel(SpectralModel):
     tag = ["BrokenPowerLawSpectralModel", "bpl"]
     index1 = Parameter("index1", 2.0)
     index2 = Parameter("index2", 2.0)
-    amplitude = Parameter("amplitude", "1e-12 cm-2 s-1 TeV-1")
+    amplitude = Parameter(
+        "amplitude", "1e-12 cm-2 s-1 TeV-1", scale_method="scale10", interp="log"
+    )
     ebreak = Parameter("ebreak", "1 TeV")
 
     @staticmethod
@@ -903,7 +998,9 @@ class SmoothBrokenPowerLawSpectralModel(SpectralModel):
     tag = ["SmoothBrokenPowerLawSpectralModel", "sbpl"]
     index1 = Parameter("index1", 2.0)
     index2 = Parameter("index2", 2.0)
-    amplitude = Parameter("amplitude", "1e-12 cm-2 s-1 TeV-1")
+    amplitude = Parameter(
+        "amplitude", "1e-12 cm-2 s-1 TeV-1", scale_method="scale10", interp="log"
+    )
     ebreak = Parameter("ebreak", "1 TeV")
     reference = Parameter("reference", "1 TeV", frozen=True)
     beta = Parameter("beta", 1, frozen=True)
@@ -918,7 +1015,7 @@ class SmoothBrokenPowerLawSpectralModel(SpectralModel):
 
 
 class PiecewiseNormSpectralModel(SpectralModel):
-    """ Piecewise spectral correction
+    """Piecewise spectral correction
        with a free normalization at each fixed energy nodes.
 
        For more information see :ref:`piecewise-norm-spectral`.
@@ -968,7 +1065,7 @@ class PiecewiseNormSpectralModel(SpectralModel):
     @property
     def norms(self):
         """Norm values"""
-        return u.Quantity(self.parameters.values)
+        return u.Quantity(self.parameters.value)
 
     def evaluate(self, energy, **norms):
         scale = interpolation_scale(scale=self._interp)
@@ -980,7 +1077,7 @@ class PiecewiseNormSpectralModel(SpectralModel):
 
     def to_dict(self, full_output=False):
         data = super().to_dict(full_output=full_output)
-        data["energy"] = {
+        data["spectral"]["energy"] = {
             "data": self.energy.data.tolist(),
             "unit": str(self.energy.unit),
         }
@@ -989,6 +1086,7 @@ class PiecewiseNormSpectralModel(SpectralModel):
     @classmethod
     def from_dict(cls, data):
         """Create model from dict"""
+        data = data["spectral"]
         energy = u.Quantity(data["energy"]["data"], data["energy"]["unit"])
         parameters = Parameters.from_dict(data["parameters"])
         return cls.from_parameters(parameters, energy=energy)
@@ -1025,7 +1123,9 @@ class ExpCutoffPowerLawSpectralModel(SpectralModel):
     tag = ["ExpCutoffPowerLawSpectralModel", "ecpl"]
 
     index = Parameter("index", 1.5)
-    amplitude = Parameter("amplitude", "1e-12 cm-2 s-1 TeV-1")
+    amplitude = Parameter(
+        "amplitude", "1e-12 cm-2 s-1 TeV-1", scale_method="scale10", interp="log"
+    )
     reference = Parameter("reference", "1 TeV", frozen=True)
     lambda_ = Parameter("lambda_", "0.1 TeV-1")
     alpha = Parameter("alpha", "1.0", frozen=True)
@@ -1081,7 +1181,7 @@ class ExpCutoffPowerLawNormSpectralModel(SpectralModel):
     tag = ["ExpCutoffPowerLawNormSpectralModel", "ecpl-norm"]
 
     index = Parameter("index", 1.5)
-    norm = Parameter("norm", 1, unit="")
+    norm = Parameter("norm", 1, unit="", interp="log")
     reference = Parameter("reference", "1 TeV", frozen=True)
     lambda_ = Parameter("lambda_", "0.1 TeV-1")
     alpha = Parameter("alpha", "1.0", frozen=True)
@@ -1114,7 +1214,9 @@ class ExpCutoffPowerLaw3FGLSpectralModel(SpectralModel):
 
     tag = ["ExpCutoffPowerLaw3FGLSpectralModel", "ecpl-3fgl"]
     index = Parameter("index", 1.5)
-    amplitude = Parameter("amplitude", "1e-12 cm-2 s-1 TeV-1")
+    amplitude = Parameter(
+        "amplitude", "1e-12 cm-2 s-1 TeV-1", scale_method="scale10", interp="log"
+    )
     reference = Parameter("reference", "1 TeV", frozen=True)
     ecut = Parameter("ecut", "10 TeV")
 
@@ -1152,7 +1254,9 @@ class SuperExpCutoffPowerLaw3FGLSpectralModel(SpectralModel):
     """
 
     tag = ["SuperExpCutoffPowerLaw3FGLSpectralModel", "secpl-3fgl"]
-    amplitude = Parameter("amplitude", "1e-12 cm-2 s-1 TeV-1")
+    amplitude = Parameter(
+        "amplitude", "1e-12 cm-2 s-1 TeV-1", scale_method="scale10", interp="log"
+    )
     reference = Parameter("reference", "1 TeV", frozen=True)
     ecut = Parameter("ecut", "10 TeV")
     index_1 = Parameter("index_1", 1.5)
@@ -1187,7 +1291,9 @@ class SuperExpCutoffPowerLaw4FGLSpectralModel(SpectralModel):
     """
 
     tag = ["SuperExpCutoffPowerLaw4FGLSpectralModel", "secpl-4fgl"]
-    amplitude = Parameter("amplitude", "1e-12 cm-2 s-1 TeV-1")
+    amplitude = Parameter(
+        "amplitude", "1e-12 cm-2 s-1 TeV-1", scale_method="scale10", interp="log"
+    )
     reference = Parameter("reference", "1 TeV", frozen=True)
     expfactor = Parameter("expfactor", "1e-2")
     index_1 = Parameter("index_1", 1.5)
@@ -1227,7 +1333,9 @@ class LogParabolaSpectralModel(SpectralModel):
 
     """
     tag = ["LogParabolaSpectralModel", "lp"]
-    amplitude = Parameter("amplitude", "1e-12 cm-2 s-1 TeV-1")
+    amplitude = Parameter(
+        "amplitude", "1e-12 cm-2 s-1 TeV-1", scale_method="scale10", interp="log"
+    )
     reference = Parameter("reference", "10 TeV", frozen=True)
     alpha = Parameter("alpha", 2)
     beta = Parameter("beta", 1)
@@ -1279,7 +1387,7 @@ class LogParabolaNormSpectralModel(SpectralModel):
     LogParabolaSpectralModel
     """
     tag = ["LogParabolaNormSpectralModel", "lp-norm"]
-    norm = Parameter("norm", 1, unit="")
+    norm = Parameter("norm", 1, unit="", interp="log")
     reference = Parameter("reference", "10 TeV", frozen=True)
     alpha = Parameter("alpha", 2)
     beta = Parameter("beta", 1)
@@ -1310,7 +1418,7 @@ class TemplateSpectralModel(SpectralModel):
     values : array
         Array with the values of the model at energies ``energy``.
     interp_kwargs : dict
-        Interpolation keyword arguments pass to `scipy.interpolate.interp1d`.
+        Interpolation keyword arguments passed to `scipy.interpolate.RegularGridInterpolator`.
         By default all values outside the interpolation range are set to zero.
         If you want to apply linear extrapolation you can pass `interp_kwargs={'fill_value':
         'extrapolate', 'kind': 'linear'}`. If you want to choose the interpolation
@@ -1322,11 +1430,15 @@ class TemplateSpectralModel(SpectralModel):
     tag = ["TemplateSpectralModel", "template"]
 
     def __init__(
-        self, energy, values, interp_kwargs=None, meta=None,
+        self,
+        energy,
+        values,
+        interp_kwargs=None,
+        meta=None,
     ):
         self.energy = energy
         self.values = u.Quantity(values, copy=False)
-        self.meta = dict() if meta is None else meta
+        self.meta = {} if meta is None else meta
         interp_kwargs = interp_kwargs or {}
         interp_kwargs.setdefault("values_scale", "log")
         interp_kwargs.setdefault("points_scale", ("log",))
@@ -1393,23 +1505,33 @@ class TemplateSpectralModel(SpectralModel):
 
     def to_dict(self, full_output=False):
         return {
-            "type": self.tag[0],
-            "energy": {
-                "data": self.energy.data.tolist(),
-                "unit": str(self.energy.unit),
-            },
-            "values": {
-                "data": self.values.data.tolist(),
-                "unit": str(self.values.unit),
-            },
+            self._type: {
+                "type": self.tag[0],
+                "energy": {
+                    "data": self.energy.data.tolist(),
+                    "unit": str(self.energy.unit),
+                },
+                "values": {
+                    "data": self.values.data.tolist(),
+                    "unit": str(self.values.unit),
+                },
+            }
         }
 
     @classmethod
     def from_dict(cls, data):
+        data = data["spectral"]
         energy = u.Quantity(data["energy"]["data"], data["energy"]["unit"])
         values = u.Quantity(data["values"]["data"], data["values"]["unit"])
         return cls(energy=energy, values=values)
 
+    @classmethod
+    def from_region_map(cls, map, **kwargs):
+        """Create model from region map"""
+        energy = map.geom.axes["energy_true"].center
+        values = map.quantity[:, 0, 0]
+        return cls(energy=energy, values=values, **kwargs)
+
 
 class ScaleSpectralModel(SpectralModel):
     """Wrapper to scale another spectral model by a norm factor.
@@ -1423,7 +1545,7 @@ class ScaleSpectralModel(SpectralModel):
     """
 
     tag = ["ScaleSpectralModel", "scale"]
-    norm = Parameter("norm", 1, unit="")
+    norm = Parameter("norm", 1, unit="", interp="log")
 
     def __init__(self, model, norm=norm.quantity):
         self.model = model
@@ -1485,24 +1607,25 @@ class EBLAbsorptionNormSpectralModel(SpectralModel):
     def to_dict(self, full_output=False):
         data = super().to_dict(full_output=full_output)
         if self.filename is None:
-            data["energy"] = {
+            data["spectral"]["energy"] = {
                 "data": self.energy.data.tolist(),
                 "unit": str(self.energy.unit),
             }
-            data["param"] = {
+            data["spectral"]["param"] = {
                 "data": self.param.data.tolist(),
                 "unit": str(self.param.unit),
             }
-            data["values"] = {
+            data["spectral"]["values"] = {
                 "data": self.data.data.tolist(),
                 "unit": str(self.data.unit),
             }
         else:
-            data["filename"] = str(self.filename)
+            data["spectral"]["filename"] = str(self.filename)
         return data
 
     @classmethod
     def from_dict(cls, data):
+        data = data["spectral"]
         redshift = [p["value"] for p in data["parameters"] if p["name"] == "redshift"][
             0
         ]
@@ -1598,7 +1721,7 @@ class EBLAbsorptionNormSpectralModel(SpectralModel):
             `Link <https://ui.adsabs.harvard.edu/abs/2010ApJ...712..238F>`__
 
         """
-        models = dict()
+        models = {}
         models["franceschini"] = "$GAMMAPY_DATA/ebl/ebl_franceschini.fits.gz"
         models["dominguez"] = "$GAMMAPY_DATA/ebl/ebl_dominguez11.fits.gz"
         models["finke"] = "$GAMMAPY_DATA/ebl/frd_abs.fits.gz"
@@ -1631,7 +1754,7 @@ class NaimaSpectralModel(SpectralModel):
         `seed_photon_fields` list defining the `radiative_model`. Default is the whole list
         of photon fields
     nested_models : dict
-        Additionnal parameters for nested models not supplied by the radiative model,
+        Additional parameters for nested models not supplied by the radiative model,
         for now this is used  only for synchrotron self-compton model
     """
 
@@ -1680,11 +1803,15 @@ class NaimaSpectralModel(SpectralModel):
             parameters.append(Parameter("B", B))
             parameters.append(Parameter("radius", radius, frozen=True))
 
+        for p in parameters:
+            p.scale_method = "scale10"
+
         self.default_parameters = Parameters(parameters)
         super().__init__()
 
     def _evaluate_ssc(
-        self, energy,
+        self,
+        energy,
     ):
         """
         Compute photon density spectrum from synchrotron emission for synchrotron self-compton model,
@@ -1784,7 +1911,7 @@ class GaussianSpectralModel(SpectralModel):
     """
 
     tag = ["GaussianSpectralModel", "gauss"]
-    norm = Parameter("norm", 1e-12 * u.Unit("cm-2 s-1"))
+    norm = Parameter("norm", 1e-12 * u.Unit("cm-2 s-1"), interp="log")
     mean = Parameter("mean", 1 * u.TeV)
     sigma = Parameter("sigma", 2 * u.TeV)
 
