@@ -6,18 +6,21 @@ from astropy import units as u
 from astropy.table import Table
 from astropy.time import Time
 from astropy.utils import lazyproperty
+from gammapy.maps import RegionNDMap, TimeMapAxis
 from gammapy.modeling import Parameter
 from gammapy.utils.random import InverseCDFSampler, get_random_state
 from gammapy.utils.scripts import make_path
 from gammapy.utils.time import time_ref_from_dict
-from .core import Model
+from .core import ModelBase
 
 
 # TODO: make this a small ABC to define a uniform interface.
-class TemporalModel(Model):
+class TemporalModel(ModelBase):
     """Temporal model base class.
     evaluates on  astropy.time.Time objects"""
 
+    _type = "temporal"
+
     def __call__(self, time):
         """Evaluate model
 
@@ -30,6 +33,10 @@ class TemporalModel(Model):
         time = u.Quantity(time.mjd, "day")
         return self.evaluate(time, **kwargs)
 
+    @property
+    def type(self):
+        return self._type
+
     @staticmethod
     def time_sum(t_min, t_max):
         """
@@ -46,13 +53,113 @@ class TemporalModel(Model):
             Summed time in the intervals.
 
         """
-        return np.sum(t_max - t_min)
+        diff = t_max - t_min
+        # TODO: this is a work-around for https://github.com/astropy/astropy/issues/10501
+        return u.Quantity(np.sum(diff.to_value("day")), "day")
+
+    def plot(self, time_range, ax=None, **kwargs):
+        """
+        Plot Temporal Model.
+
+        Parameters
+        ----------
+        time_range : `~astropy.time.Time`
+            times to plot the model
+        ax : `~matplotlib.axes.Axes`, optional
+            Axis to plot on
+        **kwargs : dict
+            Keywords forwarded to `~matplotlib.pyplot.errorbar`
+
+        Returns
+        -------
+        ax : `~matplotlib.axes.Axes`, optional
+            axis
+        """
+        time_min, time_max = time_range
+        time_axis = TimeMapAxis.from_time_bounds(
+            time_min=time_min, time_max=time_max, nbin=100
+        )
+        time_axis.time_format = "mjd"
+
+        m = RegionNDMap.create(region=None, axes=[time_axis])
+        kwargs.setdefault("marker", "None")
+        kwargs.setdefault("ls", "-")
+        kwargs.setdefault("xerr", None)
+        m.quantity = self(time_axis.time_mid)
+        ax = m.plot(ax=ax, **kwargs)
+        ax.set_ylabel("Norm / A.U.")
+        return ax
+
+    def sample_time(self, n_events, t_min, t_max, t_delta="1 s", random_state=0):
+        """Sample arrival times of events.
+
+        Parameters
+        ----------
+        n_events : int
+            Number of events to sample.
+        t_min : `~astropy.time.Time`
+            Start time of the sampling.
+        t_max : `~astropy.time.Time`
+            Stop time of the sampling.
+        t_delta : `~astropy.units.Quantity`
+            Time step used for sampling of the temporal model.
+        random_state : {int, 'random-seed', 'global-rng', `~numpy.random.RandomState`}
+            Defines random number generator initialisation.
+            Passed to `~gammapy.utils.random.get_random_state`.
+
+        Returns
+        -------
+        time : `~astropy.units.Quantity`
+            Array with times of the sampled events.
+        """
+        t_min = Time(t_min)
+        t_max = Time(t_max)
+        t_delta = u.Quantity(t_delta)
+        random_state = get_random_state(random_state)
+
+        ontime = u.Quantity((t_max - t_min).sec, "s")
+
+        time_unit = (
+            u.Unit(self.table.meta["TIMEUNIT"])
+            if hasattr(self, "table")
+            else ontime.unit
+        )
+
+        t_stop = ontime.to_value(time_unit)
+
+        # TODO: the separate time unit handling is unfortunate, but the quantity support for np.arange and np.interp
+        #  is still incomplete, refactor once we change to recent numpy and astropy versions
+        t_step = t_delta.to_value(time_unit)
+        if hasattr(self, "table"):
+            t = np.arange(0, t_stop, t_step)
+
+            pdf = self.evaluate(t)
+
+            sampler = InverseCDFSampler(pdf=pdf, random_state=random_state)
+            time_pix = sampler.sample(n_events)[0]
+            time = np.interp(time_pix, np.arange(len(t)), t) * time_unit
+
+        else:
+            t_step = (t_step * u.s).to("d")
+
+            t = Time(np.arange(t_min.mjd, t_max.mjd, t_step.value), format="mjd")
+
+            pdf = self(t)
+
+            sampler = InverseCDFSampler(pdf=pdf, random_state=random_state)
+            time_pix = sampler.sample(n_events)[0]
+            time = (
+                np.interp(time_pix, np.arange(len(t)), t.value - min(t.value))
+                * t_step.unit
+            ).to(time_unit)
+
+        return t_min + time
 
 
 class ConstantTemporalModel(TemporalModel):
     """Constant temporal model."""
 
-    tag = "ConstantTemporalModel"
+    tag = ["ConstantTemporalModel", "const"]
 
     @staticmethod
     def evaluate(time):
@@ -76,57 +183,76 @@ class ConstantTemporalModel(TemporalModel):
         """
         return (t_max - t_min) / self.time_sum(t_min, t_max)
 
+
+class LinearTemporalModel(TemporalModel):
+    """Temporal model with a linear variation.
+
+    For more information see :ref:`linear-temporal-model`.
+
+    Parameters
+    ----------
+    alpha : float
+        Constant term of the baseline flux
+    beta : `~astropy.units.Quantity`
+        Time variation coefficient of the flux
+    t_ref: `~astropy.units.Quantity`
+        The reference time in mjd. Frozen per default, at 2000-01-01.
+    """
+
+    tag = ["LinearTemporalModel", "linear"]
+
+    alpha = Parameter("alpha", 1.0, frozen=False)
+    beta = Parameter("beta", 0.0, unit="d-1", frozen=False)
+    _t_ref_default = Time("2000-01-01")
+    t_ref = Parameter("t_ref", _t_ref_default.mjd, unit="day", frozen=True)
+
     @staticmethod
-    def sample_time(n_events, t_min, t_max, random_state=0):
-        """Sample arrival times of events.
+    def evaluate(time, alpha, beta, t_ref):
+        """Evaluate at given times"""
+        return alpha + beta * (time - t_ref)
+
+    def integral(self, t_min, t_max):
+        """Evaluate the integrated flux within the given time intervals
 
         Parameters
         ----------
-        n_events : int
-            Number of events to sample.
-        t_min : `~astropy.time.Time`
-            Start time of the sampling.
-        t_max : `~astropy.time.Time`
-            Stop time of the sampling.
-        random_state : {int, 'random-seed', 'global-rng', `~numpy.random.RandomState`}
-            Defines random number generator initialisation.
-            Passed to `~gammapy.utils.random.get_random_state`.
+        t_min: `~astropy.time.Time`
+            Start times of observation
+        t_max: `~astropy.time.Time`
+            Stop times of observation
 
         Returns
         -------
-        time : `~astropy.units.Quantity`
-            Array with times of the sampled events.
+        norm : float
+            Integrated flux norm on the given time intervals
         """
-        random_state = get_random_state(random_state)
-
-        t_min = Time(t_min)
-        t_max = Time(t_max)
-
-        t_stop = (t_max - t_min).sec
-
-        time_delta = random_state.uniform(high=t_stop, size=n_events) * u.s
-
-        return t_min + time_delta
+        pars = self.parameters
+        alpha = pars["alpha"]
+        beta = pars["beta"].quantity
+        t_ref = Time(pars["t_ref"].quantity, format="mjd")
+        value = alpha * (t_max - t_min) + beta / 2.0 * (
+            (t_max - t_ref) * (t_max - t_ref) - (t_min - t_ref) * (t_min - t_ref)
+        )
+        return value / self.time_sum(t_min, t_max)
 
 
 class ExpDecayTemporalModel(TemporalModel):
-    """Temporal model with an exponential decay.
+    r"""Temporal model with an exponential decay.
 
-        ..math::
-                F(t) = exp(t - t_ref)/t0
+    .. math::
+            F(t) = exp(-(t - t_ref)/t0)
 
     Parameters
     ----------
     t0 : `~astropy.units.Quantity`
         Decay time scale
     t_ref: `~astropy.units.Quantity`
-        The reference time in mjd
+        The reference time in mjd. Frozen per default, at 2000-01-01 .
     """
 
-    tag = "ExponentialDecayTemporalModel"
+    tag = ["ExpDecayTemporalModel", "exp-decay"]
 
     t0 = Parameter("t0", "1 d", frozen=False)
-
     _t_ref_default = Time("2000-01-01")
     t_ref = Parameter("t_ref", _t_ref_default.mjd, unit="day", frozen=True)
 
@@ -158,7 +284,10 @@ class ExpDecayTemporalModel(TemporalModel):
 
 
 class GaussianTemporalModel(TemporalModel):
-    """A Gaussian temporal profile
+    r"""A Gaussian temporal profile
+
+    ..math::
+            F(t) = exp( -0.5 * \frac{ (t - t_{ref})^2 } { \sigma^2 })
 
     Parameters
     ----------
@@ -168,7 +297,7 @@ class GaussianTemporalModel(TemporalModel):
         Width of the gaussian profile.
     """
 
-    tag = "GaussianTemporalModel"
+    tag = ["GaussianTemporalModel", "gauss"]
 
     _t_ref_default = Time("2000-01-01")
     t_ref = Parameter("t_ref", _t_ref_default.mjd, unit="day", frozen=False)
@@ -215,7 +344,7 @@ class LightCurveTemplateTemporalModel(TemporalModel):
 
     The model does linear interpolation for times between the given ``(time, norm)`` values.
 
-    The implementation currently uses `scipy.interpolate.InterpolatedUnivariateSpline`,
+    The implementation currently uses `scipy.interpolate. InterpolatedUnivariateSpline`,
     using degree ``k=1`` to get linear interpolation.
     This class also contains an ``integral`` method, making the computation of
     mean fluxes for a given time interval a one-liner.
@@ -236,23 +365,27 @@ class LightCurveTemplateTemporalModel(TemporalModel):
     Show basic information about the lightcurve:
 
     >>> print(light_curve)
-    LightCurve model summary:
+    LightCurveTemplateTemporalModel model summary:
     Start time: 59000.5 MJD
     End time: 61862.5 MJD
     Norm min: 0.01551196351647377
     Norm max: 1.0
+    <BLANKLINE>
 
     Compute ``norm`` at a given time:
 
-    >>> light_curve.evaluate(46300)
-    0.49059393580053845
+    >>> light_curve.evaluate(60000)
+    array(0.01551196)
 
     Compute mean ``norm`` in a given time interval:
 
-    >>> light_curve.mean_norm_in_time_interval(46300, 46301)
+    >>> from astropy.time import Time
+    >>> times = Time([60000, 61000], format='mjd')
+    >>> light_curve.integral(times[0], times[1])
+    <Quantity 0.01721725>
     """
 
-    tag = "LightCurveTemplateTemporalModel"
+    tag = ["LightCurveTemplateTemporalModel", "template"]
 
     def __init__(self, table, filename=None):
         self.table = table
@@ -346,55 +479,119 @@ class LightCurveTemplateTemporalModel(TemporalModel):
         n2 = self._interpolator.antiderivative()(t_min.mjd)
         return u.Quantity(n1 - n2, "day") / self.time_sum(t_min, t_max)
 
-    def sample_time(self, n_events, t_min, t_max, t_delta="1 s", random_state=0):
-        """Sample arrival times of events.
+    @classmethod
+    def from_dict(cls, data):
+        return cls.read(data["temporal"]["filename"])
+
+    def to_dict(self, full_output=False):
+        """Create dict for YAML serialisation"""
+        return {self._type: {"type": self.tag[0], "filename": self.filename}}
+
+
+class PowerLawTemporalModel(TemporalModel):
+    """Temporal model with a Power Law decay.
+
+    For more information see :ref:`powerlaw-temporal-model`.
+
+    Parameters
+    ----------
+    alpha : float
+        Decay time power
+    t_ref: `~astropy.units.Quantity`
+        The reference time in mjd. Frozen by default, at 2000-01-01.
+    t0: `~astropy.units.Quantity`
+        The scaling time in mjd. Fixed by default, at 1 day.
+    """
+
+    tag = ["PowerLawTemporalModel", "powerlaw"]
+
+    alpha = Parameter("alpha", 1.0, frozen=False)
+    _t_ref_default = Time("2000-01-01")
+    t_ref = Parameter("t_ref", _t_ref_default.mjd, unit="day", frozen=True)
+    t0 = Parameter("t0", "1 d", frozen=True)
+
+    @staticmethod
+    def evaluate(time, alpha, t_ref, t0=1 * u.day):
+        """Evaluate at given times"""
+        return np.power((time - t_ref) / t0, alpha)
+
+    def integral(self, t_min, t_max):
+        """Evaluate the integrated flux within the given time intervals
 
         Parameters
         ----------
-        n_events : int
-            Number of events to sample.
-        t_min : `~astropy.time.Time`
-            Start time of the sampling.
-        t_max : `~astropy.time.Time`
-            Stop time of the sampling.
-        t_delta : `~astropy.units.Quantity`
-            Time step used for sampling of the temporal model.
-        random_state : {int, 'random-seed', 'global-rng', `~numpy.random.RandomState`}
-            Defines random number generator initialisation.
-            Passed to `~gammapy.utils.random.get_random_state`.
+        t_min: `~astropy.time.Time`
+            Start times of observation
+        t_max: `~astropy.time.Time`
+            Stop times of observation
 
         Returns
         -------
-        time : `~astropy.units.Quantity`
-            Array with times of the sampled events.
+        norm : float
+            Integrated flux norm on the given time intervals
         """
-        time_unit = getattr(u, self.table.meta["TIMEUNIT"])
+        pars = self.parameters
+        alpha = pars["alpha"].quantity
+        t0 = pars["t0"].quantity
+        t_ref = Time(pars["t_ref"].quantity, format="mjd")
+        if alpha != -1:
+            value = self.evaluate(t_max, alpha + 1.0, t_ref, t0) - self.evaluate(
+                t_min, alpha + 1.0, t_ref, t0
+            )
+            return t0 / (alpha + 1.0) * value / self.time_sum(t_min, t_max)
+        else:
+            value = np.log((t_max - t_ref) / (t_min - t_ref))
+            return t0 * value / self.time_sum(t_min, t_max)
 
-        t_min = Time(t_min)
-        t_max = Time(t_max)
-        t_delta = u.Quantity(t_delta)
-        random_state = get_random_state(random_state)
 
-        ontime = u.Quantity((t_max - t_min).sec, "s")
-        t_stop = ontime.to_value(time_unit)
+class SineTemporalModel(TemporalModel):
+    """Temporal model with a sinusoidal modulation.
 
-        # TODO: the separate time unit handling is unfortunate, but the quantity support for np.arange and np.interp
-        #  is still incomplete, refactor once we change to recent numpy and astropy versions
-        t_step = t_delta.to_value(time_unit)
-        t = np.arange(0, t_stop, t_step)
+    For more information see :ref:`sine-temporal-model`.
 
-        pdf = self.evaluate(t)
+    Parameters
+    ----------
+    amp : float
+        Amplitude of the sinusoidal function
+    t_ref: `~astropy.units.Quantity`
+        The reference time in mjd.
+    omega: `~astropy.units.Quantity`
+        Pulsation of the signal.
+    """
 
-        sampler = InverseCDFSampler(pdf=pdf, random_state=random_state)
-        time_pix = sampler.sample(n_events)[0]
-        time = np.interp(time_pix, np.arange(len(t)), t) * time_unit
+    tag = ["SineTemporalModel", "sinus"]
 
-        return t_min + time
+    amp = Parameter("amp", 1.0, frozen=False)
+    omega = Parameter("omega", "1. rad/day", frozen=False)
+    _t_ref_default = Time("2000-01-01")
+    t_ref = Parameter("t_ref", _t_ref_default.mjd, unit="day", frozen=False)
 
-    @classmethod
-    def from_dict(cls, data):
-        return cls.read(data["filename"])
+    @staticmethod
+    def evaluate(time, amp, omega, t_ref):
+        """Evaluate at given times"""
+        return 1.0 + amp * np.sin(omega * (time - t_ref))
 
-    def to_dict(self, overwrite=False):
-        """Create dict for YAML serilisation"""
-        return {"type": self.tag, "filename": self.filename}
+    def integral(self, t_min, t_max):
+        """Evaluate the integrated flux within the given time intervals
+
+        Parameters
+        ----------
+        t_min: `~astropy.time.Time`
+            Start times of observation
+        t_max: `~astropy.time.Time`
+            Stop times of observation
+
+        Returns
+        -------
+        norm : float
+            Integrated flux norm on the given time intervals
+        """
+        pars = self.parameters
+        omega = pars["omega"].quantity.to_value("rad/day")
+        amp = pars["amp"].value
+        t_ref = Time(pars["t_ref"].quantity, format="mjd")
+        value = (t_max - t_min) - amp / omega * (
+            np.sin(omega * (t_max - t_ref).to_value("day"))
+            - np.sin(omega * (t_min - t_ref).to_value("day"))
+        )
+        return value / self.time_sum(t_min, t_max)
