@@ -3,20 +3,23 @@ import pytest
 import numpy as np
 from numpy.testing import assert_allclose
 import astropy.units as u
+from astropy.coordinates import SkyCoord
 from astropy.wcs import WCS
 from regions import (
     CircleAnnulusSkyRegion,
+    CircleSkyRegion,
     EllipseSkyRegion,
     PointSkyRegion,
-    PolygonSkyRegion,
+    RectangleSkyRegion,
 )
-from gammapy.maps import Map, WcsGeom
+from gammapy.maps import Map, MapAxis, RegionGeom, WcsGeom
 from gammapy.modeling.models import (
     ConstantSpatialModel,
     DiskSpatialModel,
     GaussianSpatialModel,
     GeneralizedGaussianSpatialModel,
     PointSpatialModel,
+    Shell2SpatialModel,
     ShellSpatialModel,
     TemplateSpatialModel,
 )
@@ -105,7 +108,7 @@ def test_sky_gaussian():
 
 
 @pytest.mark.parametrize("eta", np.arange(0.1, 1.01, 0.3))
-@pytest.mark.parametrize("r_0", np.arange(0.1, 1.01, 0.3))
+@pytest.mark.parametrize("r_0", np.arange(0.01, 1.01, 0.3))
 @pytest.mark.parametrize("e", np.arange(0.0, 0.801, 0.4))
 def test_generalized_gaussian(eta, r_0, e):
     # check normalization is robust for a large set of values
@@ -113,8 +116,12 @@ def test_generalized_gaussian(eta, r_0, e):
         eta=eta, r_0=r_0 * u.deg, e=e, frame="galactic"
     )
 
+    width = np.maximum(2 * model.evaluation_radius.to_value("deg"), 0.5)
     geom = WcsGeom.create(
-        skydir=(0, 0), binsz=0.02, width=2 * model.evaluation_radius, frame="galactic",
+        skydir=(0, 0),
+        binsz=0.02,
+        width=width,
+        frame="galactic",
     )
 
     integral = model.integrate_geom(geom)
@@ -123,11 +130,13 @@ def test_generalized_gaussian(eta, r_0, e):
 
 
 def test_generalized_gaussian_io():
-    model = GeneralizedGaussianSpatialModel()
+    model = GeneralizedGaussianSpatialModel(e=0.5)
+
+    reg = model.to_region()
+    assert isinstance(reg, EllipseSkyRegion)
+    assert_allclose(reg.width.value, 1.73205, rtol=1e-5)
 
-    assert isinstance(model.to_region(), EllipseSkyRegion)
     new_model = GeneralizedGaussianSpatialModel.from_dict(model.to_dict())
-
     assert isinstance(new_model, GeneralizedGaussianSpatialModel)
 
 
@@ -143,7 +152,7 @@ def test_sky_disk():
     assert_allclose(val.value, desired)
     radius = model.evaluation_radius
     assert radius.unit == "deg"
-    assert_allclose(radius.value, r_0.value)
+    assert_allclose(radius.to_value("deg"), 2.222)
 
     # test the normalization for an elongated ellipse near the Galactic Plane
     m_geom_1 = WcsGeom.create(
@@ -163,7 +172,7 @@ def test_sky_disk():
 
     radius = model_1.evaluation_radius
     assert radius.unit == "deg"
-    assert_allclose(radius.value, r_0.value)
+    assert_allclose(radius.to_value("deg"), 11.11)
     # test rotation
     r_0 = 2 * u.deg
     semi_minor = 1 * u.deg
@@ -193,12 +202,18 @@ def test_sky_disk():
 
 def test_sky_disk_edge():
     r_0 = 2 * u.deg
-    model = DiskSpatialModel(lon_0="0 deg", lat_0="0 deg", r_0=r_0, e=0.5, phi="0 deg")
+    model = DiskSpatialModel(
+        lon_0="0 deg",
+        lat_0="0 deg",
+        r_0=r_0,
+        e=0.5,
+        phi="0 deg",
+    )
     value_center = model(0 * u.deg, 0 * u.deg)
     value_edge = model(0 * u.deg, r_0)
     assert_allclose((value_edge / value_center).to_value(""), 0.5)
 
-    edge = model.edge.quantity
+    edge = model.edge_width.value * r_0
     value_edge_pwidth = model(0 * u.deg, r_0 + edge / 2)
     assert_allclose((value_edge_pwidth / value_center).to_value(""), 0.05)
 
@@ -222,6 +237,23 @@ def test_sky_shell():
     assert isinstance(model.to_region(), CircleAnnulusSkyRegion)
 
 
+def test_sky_shell2():
+    width = 2 * u.deg
+    rad = 2 * u.deg
+    model = Shell2SpatialModel(lon_0="1 deg", lat_0="45 deg", r_0=rad + width, eta=0.5)
+    lon = [1, 2, 4] * u.deg
+    lat = 45 * u.deg
+    val = model(lon, lat)
+    assert val.unit == "deg-2"
+    desired = [55.979449, 57.831651, 94.919895]
+    assert_allclose(val.to_value("sr-1"), desired)
+    radius = model.evaluation_radius
+    assert radius.unit == "deg"
+    assert_allclose(radius.value, rad.value + width.value)
+    assert_allclose(model.r_in.value, rad.value)
+    assert isinstance(model.to_region(), CircleAnnulusSkyRegion)
+
+
 def test_sky_diffuse_constant():
     model = ConstantSpatialModel(value="42 sr-1")
     lon = [1, 2] * u.deg
@@ -236,13 +268,18 @@ def test_sky_diffuse_constant():
 
 @requires_dependency("matplotlib")
 @requires_data()
-def test_sky_diffuse_map():
+def test_sky_diffuse_map(caplog):
     filename = "$GAMMAPY_DATA/catalogs/fermi/Extended_archive_v18/Templates/RXJ1713_2016_250GeV.fits"
     model = TemplateSpatialModel.read(filename, normalize=False)
     lon = [258.5, 0] * u.deg
     lat = -39.8 * u.deg
     val = model(lon, lat)
 
+    assert "WARNING" in [_.levelname for _ in caplog.records]
+    assert "Missing spatial template unit, assuming sr^-1" in [
+        _.message for _ in caplog.records
+    ]
+
     assert val.unit == "sr-1"
     desired = [3269.178107, 0]
     assert_allclose(val.value, desired)
@@ -254,7 +291,7 @@ def test_sky_diffuse_map():
     assert radius.unit == "deg"
     assert_allclose(radius.value, 0.64, rtol=1.0e-2)
     assert model.frame == "fk5"
-    assert isinstance(model.to_region(), PolygonSkyRegion)
+    assert isinstance(model.to_region(), RectangleSkyRegion)
 
     with pytest.raises(TypeError):
         model.plot_interative()
@@ -345,3 +382,66 @@ def test_spatial_model_plot():
 
     with mpl_plot_check():
         model.plot_error(ax=ax)
+
+
+def test_integrate_region_geom():
+    center = SkyCoord("0d", "0d", frame="icrs")
+    model = GaussianSpatialModel(lon="0d", lat="0d", sigma=0.1 * u.deg, frame="icrs")
+
+    radius_large = 1 * u.deg
+    circle_large = CircleSkyRegion(center, radius_large)
+    radius_small = 0.1 * u.deg
+    circle_small = CircleSkyRegion(center, radius_small)
+
+    geom_large, geom_small = (
+        RegionGeom(region=circle_large),
+        RegionGeom(region=circle_small, binsz_wcs="0.01d"),
+    )
+
+    integral_large, integral_small = (
+        model.integrate_geom(geom_large).data,
+        model.integrate_geom(geom_small).data,
+    )
+
+    assert_allclose(integral_large[0], 1, rtol=0.001)
+    assert_allclose(integral_small[0], 0.3953, rtol=0.001)
+
+
+def test_integrate_wcs_geom():
+    center = SkyCoord("0d", "0d", frame="icrs")
+    model_0_0d = GaussianSpatialModel(
+        lon="0.234d", lat="-0.172d", sigma=1e-4 * u.deg, frame="icrs"
+    )
+
+    model_0_01d = GaussianSpatialModel(
+        lon="0.234d", lat="-0.172d", sigma=0.01 * u.deg, frame="icrs"
+    )
+    model_0_005d = GaussianSpatialModel(
+        lon="0.234d", lat="-0.172d", sigma=0.005 * u.deg, frame="icrs"
+    )
+
+    geom = WcsGeom.create(skydir=center, npix=100, binsz=0.02)
+
+    # TODO: solve issue with small radii
+    integrated_0_0d = model_0_0d.integrate_geom(geom)
+    integrated_0_01d = model_0_01d.integrate_geom(geom)
+    integrated_0_005d = model_0_005d.integrate_geom(geom)
+
+    assert_allclose(integrated_0_0d.data.sum(), 1, atol=2e-4)
+    assert_allclose(integrated_0_01d.data.sum(), 1, atol=2e-4)
+    assert_allclose(integrated_0_005d.data.sum(), 1, atol=2e-4)
+
+
+def test_integrate_geom_energy_axis():
+    center = SkyCoord("0d", "0d", frame="icrs")
+    model = GaussianSpatialModel(lon="0d", lat="0d", sigma=0.1 * u.deg, frame="icrs")
+
+    radius = 1 * u.deg
+    square = RectangleSkyRegion(center, radius, radius)
+
+    axis = MapAxis.from_energy_bounds("1 TeV", "10 TeV", nbin=10)
+    geom = RegionGeom(region=square, axes=[axis])
+
+    integral = model.integrate_geom(geom).data
+
+    assert_allclose(integral, 1, rtol=0.0001)
