@@ -2,6 +2,7 @@
 import numpy as np
 import scipy.optimize
 from gammapy.utils.interpolation import interpolate_profile
+from gammapy.utils.roots import find_roots
 from .likelihood import Likelihood
 
 __all__ = [
@@ -12,7 +13,7 @@ __all__ = [
 ]
 
 
-def optimize_scipy(parameters, function, **kwargs):
+def optimize_scipy(parameters, function, store_trace=False, **kwargs):
     method = kwargs.pop("method", "Nelder-Mead")
     pars = [par.factor for par in parameters.free_parameters]
 
@@ -22,13 +23,18 @@ def optimize_scipy(parameters, function, **kwargs):
         parmax = par.factor_max if not np.isnan(par.factor_max) else None
         bounds.append((parmin, parmax))
 
-    likelihood = Likelihood(function, parameters)
+    likelihood = Likelihood(function, parameters, store_trace)
     result = scipy.optimize.minimize(
         likelihood.fcn, pars, bounds=bounds, method=method, **kwargs
     )
 
     factors = result.x
-    info = {"success": result.success, "message": result.message, "nfev": result.nfev}
+    info = {
+        "success": result.success,
+        "message": result.message,
+        "nfev": result.nfev,
+        "trace": likelihood.trace,
+    }
     optimizer = None
 
     return factors, info, optimizer
@@ -57,45 +63,37 @@ class TSDifference:
 def _confidence_scipy_brentq(
     parameters, parameter, function, sigma, reoptimize, upper=True, **kwargs
 ):
+
     ts_diff = TSDifference(
         function, parameters, parameter, reoptimize, ts_diff=sigma ** 2
     )
 
-    kwargs.setdefault("a", parameter.factor)
+    lower_bound = parameter.factor
 
-    bound = parameter.factor_max if upper else parameter.factor_min
-
-    if np.isnan(bound):
-        bound = parameter.factor
-        if upper:
-            bound += 1e2 * parameter.error / parameter.scale
-        else:
-            bound -= 1e2 * parameter.error / parameter.scale
-
-    kwargs.setdefault("b", bound)
+    if upper:
+        upper_bound = parameter.conf_max / parameter.scale
+    else:
+        upper_bound = parameter.conf_min / parameter.scale
 
     message, success = "Confidence terminated successfully.", True
+    kwargs.setdefault("nbin", 1)
 
-    try:
-        result = scipy.optimize.brentq(ts_diff.fcn, full_output=True, **kwargs)
-    except ValueError:
+    roots, res = find_roots(
+        ts_diff.fcn, lower_bound=lower_bound, upper_bound=upper_bound, **kwargs
+    )
+    result = (roots[0], res[0])
+
+    if np.isnan(roots[0]):
         message = (
-            "Confidence estimation failed, because bracketing interval"
-            " does not contain a unique solution. Try setting the interval by hand."
+            "Confidence estimation failed. Try to set the parameter.min/max by hand."
         )
         success = False
-        result = (
-            np.nan,
-            scipy.optimize.RootResults(
-                root=np.nan, iterations=0, function_calls=0, flag=0
-            ),
-        )
 
     suffix = "errp" if upper else "errn"
 
     return {
         "nfev_" + suffix: result[1].iterations,
-        suffix: np.abs(result[0] - kwargs["a"]),
+        suffix: np.abs(result[0] - lower_bound),
         "success_" + suffix: success,
         "message_" + suffix: message,
         "stat_null": ts_diff.stat_null,
@@ -107,7 +105,7 @@ def confidence_scipy(parameters, parameter, function, sigma, reoptimize=True, **
     if len(parameters.free_parameters) <= 1:
         reoptimize = False
 
-    with parameters.restore_values:
+    with parameters.restore_status():
         result = _confidence_scipy_brentq(
             parameters=parameters,
             parameter=parameter,
@@ -118,7 +116,7 @@ def confidence_scipy(parameters, parameter, function, sigma, reoptimize=True, **
             **kwargs
         )
 
-    with parameters.restore_values:
+    with parameters.restore_status():
         result_errp = _confidence_scipy_brentq(
             parameters=parameters,
             parameter=parameter,
@@ -163,6 +161,7 @@ def stat_profile_ul_scipy(
     ul : float
         Upper limit value.
     """
+
     interp = interpolate_profile(value_scan, stat_scan, interp_scale=interp_scale)
 
     def f(x):
@@ -170,6 +169,7 @@ def stat_profile_ul_scipy(
 
     idx = np.argmin(stat_scan)
     norm_best_fit = value_scan[idx]
-    ul = scipy.optimize.brentq(f, a=norm_best_fit, b=value_scan[-1], **kwargs)
-
-    return ul
+    roots, res = find_roots(
+        f, lower_bound=norm_best_fit, upper_bound=value_scan[-1], nbin=1, **kwargs
+    )
+    return roots[0]
