@@ -85,24 +85,16 @@ def cli_analysis():
 @cli.group("download", short_help="Download datasets and notebooks")
 @click.pass_context
 def cli_download(ctx):  # noqa: D301
-    """Download notebooks, scripts and datasets.
+    """Download notebooks and datasets.
 
     \b
-    Download notebooks published as tutorials, example python scripts and the
-    related datasets needed to execute them. It is also possible to download
-    individual notebooks, scrtipts or datasets.
+    Download notebooks published in the Gammapy documentation as well as the
+    related datasets needed to execute them.
     \b
-    - The option `tutorials` will download versioned folders for the notebooks
-    and python scripts into a `gammapy-tutorials` folder created at the current
-    working directory, as well as the datasets needed to reproduce them.
+    - The option `notebooks` will download the notebook files into a `gammapy-notebooks`
+    folder created at the current working directory.
     \b
-    - The option `notebooks` will download the notebook files used in the tutorials
-    into a `gammapy-notebooks` folder created at the current working directory.
-    \b
-    - The option `scripts` will download a collection of example python scripts
-    into a `gammapy-scripts` folder created at the current working directory.
-    \b
-    - The option `datasets` will download the datasets used by Gammapy into a
+    - The option `datasets` will download the datasets used in the documentation into a
     `gammapy-datasets` folder created at the current working directory.
 
     \b
@@ -110,19 +102,21 @@ def cli_download(ctx):  # noqa: D301
     --------
 
     \b
-    $ gammapy download scripts
-    $ gammapy download notebooks
-    $ gammapy download datasets
-    $ gammapy download tutorials --release 0.8
-    $ gammapy download notebooks --src overview
-    $ gammapy download datasets --src fermi-3fhl-gc --out localfolder/
+    $ gammapy download datasets  --out localfolder
+    $ gammapy download notebooks --release 0.18 --out localfolder
     """
 
 
 @cli.group("jupyter", short_help="Perform actions on notebooks")
 @click.option("--src", default=".", help="Local folder or Jupyter notebook filename.")
+@click.option(
+    "--r",
+    default=False,
+    is_flag=True,
+    help="Apply to notebooks found recursively in folder.",
+)
 @click.pass_context
-def cli_jupyter(ctx, src):  # noqa: D301
+def cli_jupyter(ctx, src, r):  # noqa: D301
     """
     Perform a series of actions on Jupyter notebooks.
 
@@ -134,9 +128,9 @@ def cli_jupyter(ctx, src):  # noqa: D301
     --------
     \b
     $ gammapy jupyter strip
-    $ gammapy jupyter --src mynotebooks.ipynb run
-    $ gammapy jupyter --src myfolder/tutorials test
-    $ gammapy jupyter black
+    $ gammapy jupyter --src my/notebook.ipynb run
+    $ gammapy jupyter --src my/folder test
+    $ gammapy jupyter --src my/recursive/folder --r black
     """
     log = logging.getLogger(__name__)
 
@@ -146,7 +140,7 @@ def cli_jupyter(ctx, src):  # noqa: D301
         sys.exit()
 
     if path.is_dir():
-        paths = list(path.glob("*.ipynb"))
+        paths = list(path.rglob("*.ipynb")) if r else list(path.glob("*.ipynb"))
     else:
         paths = [path]
 
@@ -166,18 +160,10 @@ def add_subcommands():
 
     cli_download.add_command(cli_download_notebooks)
 
-    from .download import cli_download_scripts
-
-    cli_download.add_command(cli_download_scripts)
-
     from .download import cli_download_datasets
 
     cli_download.add_command(cli_download_datasets)
 
-    from .download import cli_download_tutorials
-
-    cli_download.add_command(cli_download_tutorials)
-
     from .jupyter import cli_jupyter_black
 
     cli_jupyter.add_command(cli_jupyter_black)
@@ -194,6 +180,10 @@ def add_subcommands():
 
     cli_jupyter.add_command(cli_jupyter_test)
 
+    from .jupyter import cli_jupyter_tar
+
+    cli_jupyter.add_command(cli_jupyter_tar)
+
     from .analysis import cli_make_config
 
     cli_analysis.add_command(cli_make_config)
