@@ -10,6 +10,8 @@ __all__ = [
     "interpolate_profile",
 ]
 
+INTERPOLATION_ORDER = {None: 0, "nearest": 0, "linear": 1, "quadratic": 2, "cubic": 3}
+
 
 class ScaledRegularGridInterpolator:
     """Thin wrapper around `scipy.interpolate.RegularGridInterpolator`.
@@ -69,7 +71,6 @@ class ScaledRegularGridInterpolator:
             kwargs.setdefault("fill_value", None)
 
         if axis is None:
-            # print(points_scaled, values_scaled)
             self._interpolate = scipy.interpolate.RegularGridInterpolator(
                 points=points_scaled, values=values_scaled, **kwargs
             )
@@ -129,6 +130,10 @@ def interpolation_scale(scale="lin"):
         return LogScale()
     elif scale == "sqrt":
         return SqrtScale()
+    elif scale == "stat-profile":
+        return StatProfileScale()
+    elif isinstance(scale, InterpolationScale):
+        return scale
     else:
         raise ValueError(f"Not a valid value scaling mode: '{scale}'.")
 
@@ -138,7 +143,7 @@ class InterpolationScale:
 
     def __call__(self, values):
         if hasattr(self, "_unit"):
-            values = values.to_value(self._unit)
+            values = u.Quantity(values, copy=False).to_value(self._unit)
         else:
             if isinstance(values, u.Quantity):
                 self._unit = values.unit
@@ -146,7 +151,7 @@ class InterpolationScale:
         return self._scale(values)
 
     def inverse(self, values):
-        values = self._inverse(self, values)
+        values = self._inverse(values)
         if hasattr(self, "_unit"):
             return u.Quantity(values, self._unit, copy=False)
         else:
@@ -162,10 +167,10 @@ class LogScale(InterpolationScale):
         values = np.clip(values, self.tiny, np.inf)
         return np.log(values)
 
-    @staticmethod
-    def _inverse(self, values):
+    @classmethod
+    def _inverse(cls, values):
         output = np.exp(values)
-        return np.where(abs(output) - self.tiny <= self.tiny, 0, output)
+        return np.where(abs(output) - cls.tiny <= cls.tiny, 0, output)
 
 
 class SqrtScale(InterpolationScale):
@@ -176,8 +181,24 @@ class SqrtScale(InterpolationScale):
         sign = np.sign(values)
         return sign * np.sqrt(sign * values)
 
-    @staticmethod
-    def _inverse(self, values):
+    @classmethod
+    def _inverse(cls, values):
+        return np.power(values, 2)
+
+
+class StatProfileScale(InterpolationScale):
+    """Sqrt scaling"""
+
+    def __init__(self, axis=0):
+        self.axis = axis
+
+    def _scale(self, values):
+        values = np.sign(np.gradient(values, axis=self.axis)) * values
+        sign = np.sign(values)
+        return sign * np.sqrt(sign * values)
+
+    @classmethod
+    def _inverse(cls, values):
         return np.power(values, 2)
 
 
@@ -188,8 +209,8 @@ class LinearScale(InterpolationScale):
     def _scale(values):
         return values
 
-    @staticmethod
-    def _inverse(self, values):
+    @classmethod
+    def _inverse(cls, values):
         return values
 
 
